@@ -1,4 +1,3 @@
-#include <iostream>
 #include <fstream>
 
 #include <lfortran/fortran_evaluator.h>
@@ -18,7 +17,7 @@
 #include <lfortran/pickle.h>
 #include <libasr/pickle.h>
 #include <libasr/utils.h>
-#include "libasr/asr_lookup_name.cpp"
+#include <libasr/asr_lookup_name.h>
 
 
 #ifdef HAVE_LFORTRAN_LLVM
@@ -97,7 +96,7 @@ Result<FortranEvaluator::EvalResult> FortranEvaluator::evaluate(
     }
 
     // AST -> ASR
-    Result<ASR::TranslationUnit_t*> res2 = get_asr3(*ast, diagnostics);
+    Result<ASR::TranslationUnit_t*> res2 = get_asr3(*ast, diagnostics, lm);
     ASR::TranslationUnit_t* asr;
     if (res2.ok) {
         asr = res2.result;
@@ -273,7 +272,7 @@ Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr2(
         return res.error;
     }
     // AST -> ASR
-    Result<ASR::TranslationUnit_t*> res2 = get_asr3(*ast, diagnostics);
+    Result<ASR::TranslationUnit_t*> res2 = get_asr3(*ast, diagnostics, lm);
     if (res2.ok) {
         return res2.result;
     } else {
@@ -283,7 +282,7 @@ Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr2(
 }
 
 Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr3(
-    LFortran::AST::TranslationUnit_t &ast, diag::Diagnostics &diagnostics)
+    LFortran::AST::TranslationUnit_t &ast, diag::Diagnostics &diagnostics, LCompilers::LocationManager &lm)
 {
     ASR::TranslationUnit_t* asr;
     // AST -> ASR
@@ -295,7 +294,7 @@ Result<ASR::TranslationUnit_t*> FortranEvaluator::get_asr3(
         symbol_table->mark_all_variables_external(al);
     }
     auto res = LFortran::ast_to_asr(al, ast, diagnostics, symbol_table,
-        compiler_options.symtab_only, compiler_options);
+        compiler_options.symtab_only, compiler_options, lm);
     if (res.ok) {
         asr = res.result;
     } else {
@@ -534,32 +533,7 @@ Result<std::string> FortranEvaluator::get_julia(const std::string &code,
     }
 }
 
-Result<std::string> FortranEvaluator::get_mlir(const std::string &code,
-    LocationManager &lm, diag::Diagnostics &diagnostics)
-{
-    // Src -> AST -> ASR -> MLIR
-    SymbolTable *old_symbol_table = symbol_table;
-    symbol_table = nullptr;
-    Result<ASR::TranslationUnit_t*> asr = get_asr2(code, lm, diagnostics);
-    symbol_table = old_symbol_table;
-    if (!asr.ok) {
-        return asr.error;
-    }
-#ifdef HAVE_LFORTRAN_MLIR
-    Result<std::unique_ptr<MLIRModule>> res = asr_to_mlir(al, *asr.result,
-        diagnostics);
-    if (res.ok) {
-        return res.result->str();
-    } else {
-        LCOMPILERS_ASSERT(diagnostics.has_error())
-        return res.error;
-    }
-#else
-    throw LCompilersException("MLIR is not enabled");
-#endif
-}
-
-Result<std::unique_ptr<MLIRModule>> FortranEvaluator::get_mlir2(
+Result<std::unique_ptr<MLIRModule>> FortranEvaluator::get_mlir(
 #ifdef HAVE_LFORTRAN_MLIR
         ASR::TranslationUnit_t &asr, diag::Diagnostics &diagnostics
 #else
@@ -569,6 +543,9 @@ Result<std::unique_ptr<MLIRModule>> FortranEvaluator::get_mlir2(
 #ifdef HAVE_LFORTRAN_MLIR
     // ASR -> MLIR
     std::unique_ptr<LCompilers::MLIRModule> m;
+    LCompilers::PassManager pass_manager;
+    pass_manager.use_default_passes();
+    pass_manager.apply_passes(al, &asr, compiler_options.po, diagnostics);
     Result<std::unique_ptr<MLIRModule>> res = asr_to_mlir(al, asr, diagnostics);
     if (res.ok) {
         m = std::move(res.result);
