@@ -31,9 +31,9 @@ define i64 @f1()
     ret i64 4
 }
     )""");
-    CHECK(e.int64fn("f1") == 4);
+    CHECK(e.execfn<int64_t>("f1") == 4);
     e.add_module("");
-//    CHECK(e.int64fn("f1") == 4);
+//    CHECK(e.execfn<int64_t>("f1") == 4);
 
 /*
     e.add_module(R"""(
@@ -42,9 +42,9 @@ define i64 @f1()
     ret i64 5
 }
     )""");
-    CHECK(e.int64fn("f1") == 5);
+    CHECK(e.execfn<int64_t>("f1") == 5);
     e.add_module("");
-    CHECK(e.int64fn("f1") == 5);
+    CHECK(e.execfn<int64_t>("f1") == 5);
 */
 }
 
@@ -79,7 +79,7 @@ define i64 @f1()
     ret i64 %1
 }
     )""");
-    CHECK(e.int64fn("f1") == 4);
+    CHECK(e.execfn<int64_t>("f1") == 4);
 
     e.add_module(R"""(
 @count = external global i64
@@ -90,7 +90,7 @@ define i64 @f2()
     ret i64 %1
 }
     )""");
-    CHECK(e.int64fn("f2") == 4);
+    CHECK(e.execfn<int64_t>("f2") == 4);
 
     CHECK_THROWS_AS(e.add_module(R"""(
 define i64 @f3()
@@ -125,12 +125,12 @@ define void @inc()
     ret void
 }
     )""");
-    CHECK(e.int64fn("f1") == 5);
+    CHECK(e.execfn<int64_t>("f1") == 5);
 
-//    e.voidfn("inc");
-//    CHECK(e.int64fn("f1") == 6);
-/*    e.voidfn("inc");
-    CHECK(e.int64fn("f1") == 7);
+//    e.execfn<void>("inc");
+//    CHECK(e.execfn<int64_t>("f1") == 6);
+/*    e.execfn<void>("inc");
+    CHECK(e.execfn<int64_t>("f1") == 7);
     */
 /*
     e.add_module(R"""(
@@ -144,13 +144,13 @@ define void @inc2()
     ret void
 }
     )""");
-    CHECK(e.int64fn("f1") == 7);
-    e.voidfn("inc2");
-    CHECK(e.int64fn("f1") == 9);
-    e.voidfn("inc2");
-    CHECK(e.int64fn("f1") == 11);
-    e.voidfn("inc2");
-    CHECK(e.int64fn("f1") == 13);
+    CHECK(e.execfn<int64_t>("f1") == 7);
+    e.execfn<void>("inc2");
+    CHECK(e.execfn<int64_t>("f1") == 9);
+    e.execfn<void>("inc2");
+    CHECK(e.execfn<int64_t>("f1") == 11);
+    e.execfn<void>("inc2");
+    CHECK(e.execfn<int64_t>("f1") == 13);
 
     // Test that we can have another independent LLVMEvaluator and use both at
     // the same time:
@@ -173,19 +173,19 @@ define void @inc()
 }
     )""");
 
-    CHECK(e2.int64fn("f1") == 5);
-    e2.voidfn("inc");
-    CHECK(e2.int64fn("f1") == 6);
-    e2.voidfn("inc");
-    CHECK(e2.int64fn("f1") == 7);
+    CHECK(e2.execfn<int64_t>("f1") == 5);
+    e2.execfn<void>("inc");
+    CHECK(e2.execfn<int64_t>("f1") == 6);
+    e2.execfn<void>("inc");
+    CHECK(e2.execfn<int64_t>("f1") == 7);
 
-    CHECK(e.int64fn("f1") == 12);
-    e2.voidfn("inc");
-    CHECK(e2.int64fn("f1") == 8);
-    CHECK(e.int64fn("f1") == 12);
-    e.voidfn("inc2");
-    CHECK(e2.int64fn("f1") == 8);
-    CHECK(e.int64fn("f1") == 14);
+    CHECK(e.execfn<int64_t>("f1") == 12);
+    e2.execfn<void>("inc");
+    CHECK(e2.execfn<int64_t>("f1") == 8);
+    CHECK(e.execfn<int64_t>("f1") == 12);
+    e.execfn<void>("inc2");
+    CHECK(e2.execfn<int64_t>("f1") == 8);
+    CHECK(e.execfn<int64_t>("f1") == 14);
 */
 }
 
@@ -208,11 +208,11 @@ define void @inc()
     ret void
 }
 )""");
-    CHECK(e.int64fn("f1") == 5);
-/*    e.voidfn("inc");
-    CHECK(e.int64fn("f1") == 6);
-    e.voidfn("inc");
-    CHECK(e.int64fn("f1") == 7);
+    CHECK(e.execfn<int64_t>("f1") == 5);
+/*    e.execfn<void>("inc");
+    CHECK(e.execfn<int64_t>("f1") == 6);
+    e.execfn<void>("inc");
+    CHECK(e.execfn<int64_t>("f1") == 7);
     */
 /*
     e.add_module(R"""(
@@ -225,13 +225,13 @@ define void @inc2()
     ret void
 }
 )""");
-    CHECK(e.int64fn("f1") == 7);
-    e.voidfn("inc2");
-    CHECK(e.int64fn("f1") == 9);
-    e.voidfn("inc");
-    CHECK(e.int64fn("f1") == 10);
-    e.voidfn("inc2");
-    CHECK(e.int64fn("f1") == 12);
+    CHECK(e.execfn<int64_t>("f1") == 7);
+    e.execfn<void>("inc2");
+    CHECK(e.execfn<int64_t>("f1") == 9);
+    e.execfn<void>("inc");
+    CHECK(e.execfn<int64_t>("f1") == 10);
+    e.execfn<void>("inc2");
+    CHECK(e.execfn<int64_t>("f1") == 12);
 
     CHECK_THROWS_AS(e.add_module(R"""(
 define void @inc2()
@@ -284,7 +284,7 @@ define i64 @f()
     ret i64 %r
 }
     )""");
-    CHECK(e.int64fn("f") == 6);
+    CHECK(e.execfn<int64_t>("f") == 6);
 }
 
 TEST_CASE("llvm array 2") {
@@ -329,7 +329,7 @@ define i64 @f()
     ret i64 %r
 }
     )""");
-    //CHECK(e.int64fn("f") == 6);
+    //CHECK(e.execfn<int64_t>("f") == 6);
 }
 
 int f(int a, int b) {
@@ -353,7 +353,7 @@ define i64 @f1()
     ret i64 %r
 }
     )""");
-    CHECK(e.int64fn("f1") == 5);
+    CHECK(e.execfn<int64_t>("f1") == 5);
 }
 
 
@@ -375,9 +375,10 @@ end function)";
 
     // AST -> ASR
     LCompilers::SymbolTable::reset_global_counter();
+    LCompilers::LocationManager lm;
     LCompilers::ASR::TranslationUnit_t* asr = TRY(LCompilers::LFortran::ast_to_asr(al, *tu,
-        diagnostics, nullptr, false, compiler_options));
-    CHECK(LCompilers::pickle(*asr) == "(TranslationUnit (SymbolTable 1 {f: (Function (SymbolTable 2 {f: (Variable 2 f [] ReturnVar () () Default (Integer 4) () Source Public Required .false.)}) f (FunctionType [] (Integer 4) Source Implementation () .false. .false. .false. .false. .false. [] .false.) [] [] [(Assignment (Var 2 f) (IntegerConstant 5 (Integer 4)) ())] (Var 2 f) Public .false. .false. ())}) [])");
+        diagnostics, nullptr, false, compiler_options, lm));
+    CHECK(LCompilers::pickle(*asr) == "(TranslationUnit (SymbolTable 1 {f: (Function (SymbolTable 2 {f: (Variable 2 f [] ReturnVar () () Default (Integer 4) () Source Public Required .false. .false.)}) f (FunctionType [] (Integer 4) Source Implementation () .false. .false. .false. .false. .false. [] .false.) [] [] [(Assignment (Var 2 f) (IntegerConstant 5 (Integer 4) Decimal) ())] (Var 2 f) Public .false. .false. ())}) [])");
 
     // ASR -> LLVM
     LCompilers::LLVMEvaluator e;
@@ -396,7 +397,7 @@ end function)";
 
     // LLVM -> Machine code -> Execution
     e.add_module(std::move(m));
-    CHECK(e.int32fn("f") == 5);
+    CHECK(e.execfn<int32_t>("f") == 5);
 }
 
 TEST_CASE("ASR -> LLVM 2") {
@@ -416,9 +417,10 @@ end function)";
     CHECK(LCompilers::LFortran::pickle(*ast) == "(Function f [] [] () () () [] [] [] [(Declaration (AttrType TypeInteger [] () () None) [] [(f [] [] () () None ())] ())] [(Assignment 0 f 4 ())] [] [])");
 
     // AST -> ASR
+    LCompilers::LocationManager lm;
     LCompilers::ASR::TranslationUnit_t* asr = TRY(LCompilers::LFortran::ast_to_asr(al, *tu,
-        diagnostics, nullptr, false, compiler_options));
-    CHECK(LCompilers::pickle(*asr) == "(TranslationUnit (SymbolTable 3 {f: (Function (SymbolTable 4 {f: (Variable 4 f [] ReturnVar () () Default (Integer 4) () Source Public Required .false.)}) f (FunctionType [] (Integer 4) Source Implementation () .false. .false. .false. .false. .false. [] .false.) [] [] [(Assignment (Var 4 f) (IntegerConstant 4 (Integer 4)) ())] (Var 4 f) Public .false. .false. ())}) [])");
+        diagnostics, nullptr, false, compiler_options, lm));
+    CHECK(LCompilers::pickle(*asr) == "(TranslationUnit (SymbolTable 3 {f: (Function (SymbolTable 4 {f: (Variable 4 f [] ReturnVar () () Default (Integer 4) () Source Public Required .false. .false.)}) f (FunctionType [] (Integer 4) Source Implementation () .false. .false. .false. .false. .false. [] .false.) [] [] [(Assignment (Var 4 f) (IntegerConstant 4 (Integer 4) Decimal) ())] (Var 4 f) Public .false. .false. ())}) [])");
     // ASR -> LLVM
     LCompilers::LLVMEvaluator e;
     LCompilers::PassManager lpm;
@@ -433,7 +435,7 @@ end function)";
 
     // LLVM -> Machine code -> Execution
     e.add_module(std::move(m));
-    CHECK(e.int32fn("f") == 4);
+    CHECK(e.execfn<int32_t>("f") == 4);
 }
 
 TEST_CASE("FortranEvaluator 1") {
@@ -664,7 +666,7 @@ define float @f()
     ret float %r
 }
     )""");
-    CHECK(std::abs(e.floatfn("f") - 8) < 1e-6);
+    CHECK(std::abs(e.execfn<float>("f") - 8) < 1e-6);
 }
 
 // Tests passing the complex struct by value
@@ -703,7 +705,7 @@ define float @f()
     ret float %r
 }
     )""");
-//    CHECK(std::abs(e.floatfn("f") - 8) < 1e-6);
+//    CHECK(std::abs(e.execfn<float>("f") - 8) < 1e-6);
 }
 
 // Tests passing boolean by reference
@@ -733,7 +735,7 @@ define i1 @b()
     ret i1 %r
 }
     )""");
-    CHECK(e.boolfn("b") == false);
+    CHECK(e.execfn<bool>("b") == false);
 }
 
 // Tests passing boolean by value
@@ -763,7 +765,7 @@ define i1 @b()
     ret i1 %r
 }
     )""");
-    CHECK(e.boolfn("b") == false);
+    CHECK(e.execfn<bool>("b") == false);
 }
 
 // Tests pointers
@@ -784,7 +786,7 @@ define i64 @f()
     ret i64 %raddr
 }
     )""");
-    int64_t r = e.int64fn("f");
+    int64_t r = e.execfn<int64_t>("f");
     CHECK(r != 8);
     int64_t *p = (int64_t*)r;
     CHECK(*p == 8);
@@ -804,7 +806,7 @@ define i64 @f()
     ret i64 %raddr
 }
     )""");
-    int64_t r = e.int64fn("f");
+    int64_t r = e.execfn<int64_t>("f");
     float *p = (float *)r;
     CHECK(std::abs(*p - 8) < 1e-6);
 }
@@ -845,7 +847,7 @@ define float @f()
     ret float %ret
 }
     )""");
-    float r = e.floatfn("f");
+    float r = e.execfn<float>("f");
     CHECK(std::abs(r - 8) < 1e-6);
 }
 
@@ -882,7 +884,7 @@ define float @f()
     ret float %ret
 }
     )""");
-    float r = e.floatfn("f");
+    float r = e.execfn<float>("f");
     CHECK(std::abs(r - 8) < 1e-6);
 }
 
@@ -961,6 +963,93 @@ TEST_CASE("FortranEvaluator 9 double complex") {
     }
 }
 
+TEST_CASE("FortranEvaluator logical 1") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2("logical :: i");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("i = .true.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("i");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(r.result.b);
+}
+
+TEST_CASE("FortranEvaluator logical 2") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2("logical :: i");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("i = .false.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("i");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(!r.result.b);
+}
+
+TEST_CASE("FortranEvaluator logical 3") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2(".false. .or. .true.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(r.result.b);
+    r = e.evaluate2(".false. .and. .true.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(!r.result.b);
+    r = e.evaluate2("logical :: i");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("i = .false.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("i .or. .true.");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(r.result.b);
+}
+
+TEST_CASE("FortranEvaluator logical 4") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2(R"(
+function is_even(n) result(result)
+integer, intent(in) :: n
+logical :: result
+result = mod(n, 2) == 0
+end function is_even
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("is_even(3)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(!r.result.b);
+    r = e.evaluate2("is_even(4)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::boolean);
+    CHECK(r.result.b);
+}
+
 TEST_CASE("FortranEvaluator integer kind 1") {
     CompilerOptions cu;
     cu.interactive = true;
@@ -1011,6 +1100,23 @@ TEST_CASE("FortranEvaluator Array 1") {
     CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
 }
 
+TEST_CASE("FortranEvaluator Array 2") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2("integer :: x(3)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("x = 5");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("print *, x");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+}
+
 TEST_CASE("FortranEvaluator re-declaration 1") {
     CompilerOptions cu;
     cu.interactive = true;
@@ -1077,6 +1183,143 @@ end function
 */
 }
 
+TEST_CASE("FortranEvaluator asr verify 1") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2(R"(
+pure function double(x) result(r)
+    integer, intent(in) :: x
+    integer :: r
+    r = 2 * x
+end function double
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2(R"(
+subroutine s(n, x)
+    integer, intent(in) :: n
+    integer, intent(inout) :: x
+    x = double(n)
+end subroutine s
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("integer :: x");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("x = double(1)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("x");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::integer4);
+    CHECK(r.result.i32 == 2);
+    r = e.evaluate2("call s(x, x)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("x");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::integer4);
+    CHECK(r.result.i32 == 4);
+}
+
+TEST_CASE("FortranEvaluator asr verify 2") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2(R"(
+pure function id(x) result(r)
+    integer, intent(in) :: x
+    integer :: r
+    r = x
+end function id
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2(R"(
+subroutine sa(l, a)
+    integer, intent(in) :: l
+    integer, intent(inout) :: a(id(l))
+    
+    integer :: i
+    
+    do i = 1, size(a)
+        a(i) = a(i) + 1
+    end do
+end subroutine sa
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("integer :: arr(3)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("arr = 0");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("call sa(3, arr)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::statement);
+    r = e.evaluate2("arr(1)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::integer4);
+    CHECK(r.result.i32 == 1);
+    r = e.evaluate2("arr(2)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::integer4);
+    CHECK(r.result.i32 == 1);
+    r = e.evaluate2("arr(3)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::integer4);
+    CHECK(r.result.i32 == 1);
+}
+
+TEST_CASE("FortranEvaluator asr verify 3") {
+    CompilerOptions cu;
+    cu.interactive = true;
+    cu.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
+    FortranEvaluator e(cu);
+    LCompilers::Result<FortranEvaluator::EvalResult>
+    r = e.evaluate2(R"(
+function add(x, y) result(r)
+    real, intent(in) :: x
+    real, intent(in) :: y
+    real :: r
+    r = x + y
+end function add
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2(R"(
+function sub(x, y) result(r)
+    real, intent(in) :: x
+    real, intent(in) :: y
+    real :: r
+    r = add(x, -y)
+end function sub
+)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::none);
+    r = e.evaluate2("add(2.0, 3.0)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::real4);
+    CHECK(r.result.f32 == 5.0);
+    r = e.evaluate2("sub(2.0, 3.0)");
+    CHECK(r.ok);
+    CHECK(r.result.type == FortranEvaluator::EvalResult::real4);
+    CHECK(r.result.f32 == -1.0);
+}
+
+TEST_CASE("llvm ir 1") {
+    LCompilers::LLVMEvaluator e;
+    CHECK_THROWS_AS(e.parse_module2("", "src/lfortran/tests/ir.ll"), LCompilers::LCompilersException);
+    CHECK_THROWS_WITH(e.parse_module2("", "src/lfortran/tests/ir.ll"), "parse_module(): Invalid LLVM IR");
+}
+
 // This test does not work on Windows yet
 // https://github.com/lfortran/lfortran/issues/913
 #if !defined(_WIN32)
