@@ -39,7 +39,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
@@ -50,7 +50,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
@@ -61,7 +61,7 @@ class ASRBuilder {
             ASR::ttype_t *type, ASR::intentType intent,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
-            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
             intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_or_overwrite_symbol(s2c(al, var_name), sym);
@@ -96,17 +96,17 @@ class ASRBuilder {
         false, nullptr, 0, false, false, false));
 
     // Types -------------------------------------------------------------------
-    #define int8         TYPE(ASR::make_Integer_t(al, loc, 1))
-    #define int16        TYPE(ASR::make_Integer_t(al, loc, 2))
-    #define int32        TYPE(ASR::make_Integer_t(al, loc, 4))
-    #define int64        TYPE(ASR::make_Integer_t(al, loc, 8))
-    #define real32       TYPE(ASR::make_Real_t(al, loc, 4))
-    #define real64       TYPE(ASR::make_Real_t(al, loc, 8))
-    #define complex32    TYPE(ASR::make_Complex_t(al, loc, 4))
-    #define complex64    TYPE(ASR::make_Complex_t(al, loc, 8))
-    #define logical      TYPE(ASR::make_Logical_t(al, loc, 4))
-    #define character(x) TYPE(ASR::make_Character_t(al, loc, 1, x, nullptr))
-    #define List(x)      TYPE(ASR::make_List_t(al, loc, x))
+    #define int8         ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 1))
+    #define int16        ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 2))
+    #define int32        ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4))
+    #define int64        ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 8))
+    #define real32       ASRUtils::TYPE(ASR::make_Real_t(al, loc, 4))
+    #define real64       ASRUtils::TYPE(ASR::make_Real_t(al, loc, 8))
+    #define complex32    ASRUtils::TYPE(ASR::make_Complex_t(al, loc, 4))
+    #define complex64    ASRUtils::TYPE(ASR::make_Complex_t(al, loc, 8))
+    #define logical      ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4))
+    #define character(x) ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, x, nullptr, ASR::string_physical_typeType::PointerString))
+    #define List(x)      ASRUtils::TYPE(ASR::make_List_t(al, loc, x))
 
     ASR::ttype_t *Tuple(std::vector<ASR::ttype_t*> tuple_type) {
         Vec<ASR::ttype_t*> m_tuple_type; m_tuple_type.reserve(al, 3);
@@ -143,11 +143,53 @@ class ASRBuilder {
     }
 
     ASR::expr_t* ArrayUBound(ASR::expr_t* x, int64_t dim) {
-        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::UBound, nullptr));
+        ASR::expr_t* value = nullptr;
+        ASR::ttype_t* type = ASRUtils::expr_type(x);
+        if ( ASRUtils::is_array(type) ) {
+            ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(ASRUtils::type_get_past_pointer(ASRUtils::type_get_past_allocatable(type)));
+            ASR::dimension_t* dims = array_type->m_dims;
+            ASRUtils::extract_dimensions_from_ttype(type, dims);
+            int new_dim = dim - 1;
+            if( dims[new_dim].m_start && dims[new_dim].m_length ) {
+                ASR::expr_t* start = ASRUtils::expr_value(dims[new_dim].m_start);
+                ASR::expr_t* length = ASRUtils::expr_value(dims[new_dim].m_length);
+                if( ASRUtils::is_value_constant(start) &&
+                ASRUtils::is_value_constant(length) ) {
+                    int64_t const_lbound = -1;
+                    if( !ASRUtils::extract_value(start, const_lbound) ) {
+                        LCOMPILERS_ASSERT(false);
+                    }
+                    int64_t const_length = -1;
+                    if( !ASRUtils::extract_value(length, const_length) ) {
+                        LCOMPILERS_ASSERT(false);
+                    }
+                    value = i32(const_lbound + const_length - 1);
+                }
+            }
+        }
+        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::UBound, value));
     }
 
     ASR::expr_t* ArrayLBound(ASR::expr_t* x, int64_t dim) {
-        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::LBound, nullptr));
+        ASR::expr_t* value = nullptr;
+        ASR::ttype_t* type = ASRUtils::expr_type(x);
+        if ( ASRUtils::is_array(type) ) {
+            ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(ASRUtils::type_get_past_pointer(ASRUtils::type_get_past_allocatable(type)));
+            ASR::dimension_t* dims = array_type->m_dims;
+            ASRUtils::extract_dimensions_from_ttype(type, dims);
+            int new_dim = dim - 1;
+            if( dims[new_dim].m_start ) {
+                ASR::expr_t* start = ASRUtils::expr_value(dims[new_dim].m_start);
+                if( ASRUtils::is_value_constant(start) ) {
+                    int64_t const_lbound = -1;
+                    if( !ASRUtils::extract_value(start, const_lbound) ) {
+                        LCOMPILERS_ASSERT(false);
+                    }
+                    value = i32(const_lbound);
+                }
+            }
+        }
+        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::LBound, value));
     }
 
     inline ASR::expr_t* i_t(int64_t x, ASR::ttype_t* t) {
@@ -259,26 +301,94 @@ class ASRBuilder {
 
     // Cast --------------------------------------------------------------------
 
+    #define avoid_cast(x, t) if( ASRUtils::extract_kind_from_ttype_t(t) <= \
+        ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(x)) ) { \
+        return x; \
+    } \
+
     inline ASR::expr_t* r2i_t(ASR::expr_t* x, ASR::ttype_t* t) {
-        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::RealToInteger, t, nullptr));
+        ASR::expr_t* value = ASRUtils::expr_value(x);
+        if ( value != nullptr ) {
+            double val = ASR::down_cast<ASR::RealConstant_t>(value)->m_r;
+            value = i_t(val, t);
+        }
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::RealToInteger, t, value));
+    }
+
+    inline ASR::expr_t* c2i_t(ASR::expr_t* x, ASR::ttype_t* t) {
+        // TODO: handle value
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToInteger, t, nullptr));
     }
 
     inline ASR::expr_t* i2r_t(ASR::expr_t* x, ASR::ttype_t* t) {
-        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::IntegerToReal, t, nullptr));
+        ASR::expr_t* value = ASRUtils::expr_value(x);
+        if ( value != nullptr ) {
+            int64_t val = ASR::down_cast<ASR::IntegerConstant_t>(value)->m_n;
+            value = f_t(val, t);
+        }
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::IntegerToReal, t, value));
     }
 
     inline ASR::expr_t* i2i_t(ASR::expr_t* x, ASR::ttype_t* t) {
-        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::IntegerToInteger, t, nullptr));
+        // avoid_cast(x, t); // TODO: adding this makes intrinsics_61 fail, that shall not happen, add a flag for force casting
+        ASR::expr_t* value = ASRUtils::expr_value(x);
+        if ( value != nullptr ) {
+            int64_t val = ASR::down_cast<ASR::IntegerConstant_t>(value)->m_n;
+            value = i_t(val, t);
+        }
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::IntegerToInteger, t, value));
     }
 
     inline ASR::expr_t* r2r_t(ASR::expr_t* x, ASR::ttype_t* t) {
-        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::RealToReal, t, nullptr));
+        int kind_x = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(x));
+        int kind_t = ASRUtils::extract_kind_from_ttype_t(t);
+        if (kind_x == kind_t) {
+            return x;
+        }
+        ASR::expr_t* value = ASRUtils::expr_value(x);
+        if ( value != nullptr ) {
+            double val = ASR::down_cast<ASR::RealConstant_t>(value)->m_r;
+            value = f_t(val, t);
+        }
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::RealToReal, t, value));
     }
 
     inline ASR::expr_t* c2r_t(ASR::expr_t* x, ASR::ttype_t* t) {
+        // TODO: handle value
         return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToReal, t, nullptr));
     }
 
+    inline ASR::expr_t* t2t(ASR::expr_t* x, ASR::ttype_t* t1, ASR::ttype_t* t2) {
+        // TODO: improve this function to handle all types
+        if (ASRUtils::is_real(*t1)) {
+            if (ASRUtils::is_real(*t2)) {
+                return r2r_t(x, t2);
+            } else if (ASRUtils::is_integer(*t2)) {
+                return r2i_t(x, t2);
+            } else {
+                throw LCompilersException("Type not supported");
+            }
+        } else if (ASRUtils::is_integer(*t1)) {
+            if (ASRUtils::is_real(*t2)) {
+                return i2r_t(x, t2);
+            } else if (ASRUtils::is_integer(*t2)) {
+                return i2i_t(x, t2);
+            } else {
+                throw LCompilersException("Type not supported");
+            }
+        } else if (ASRUtils::is_complex(*t1)) {
+            if (ASRUtils::is_real(*t2)) {
+                return c2r_t(x, t2);
+            } else if (ASRUtils::is_integer(*t2)) {
+                return c2i_t(x, t2);
+            } else {
+                throw LCompilersException("Type not supported");
+            }
+        } else {
+            throw LCompilersException("Type not supported");
+        }
+    }
+
     // Binop -------------------------------------------------------------------
 
     inline ASR::expr_t* BitRshift(ASR::expr_t* n, ASR::expr_t* bits, ASR::ttype_t* t) {
@@ -302,7 +412,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -321,7 +431,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -340,7 +450,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -357,7 +467,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(x)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(x)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -376,7 +486,7 @@ class ASRBuilder {
                 return EXPR(ASR::make_RealBinOp_t(al, loc, left,
                     ASR::binopType::Add, right, type, nullptr));
             }
-            case ASR::ttypeType::Character : {
+            case ASR::ttypeType::String : {
                 return EXPR(ASR::make_StringConcat_t(al, loc, left,
                     right, type, nullptr));
             }
@@ -386,7 +496,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -411,7 +521,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -423,12 +533,26 @@ class ASRBuilder {
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
+                int64_t left_value = 0, right_value = 0;
+                ASR::expr_t* value = nullptr;
+                if( ASRUtils::extract_value(left, left_value) &&
+                    ASRUtils::extract_value(right, right_value) ) {
+                    int64_t mul_value = left_value * right_value;
+                    value = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, mul_value, type));
+                }
                 return EXPR(ASR::make_IntegerBinOp_t(al, loc, left,
-                    ASR::binopType::Mul, right, type, nullptr));
+                    ASR::binopType::Mul, right, type, value));
             }
             case ASR::ttypeType::Real: {
+                double left_value = 0, right_value = 0;
+                ASR::expr_t* value = nullptr;
+                if( ASRUtils::extract_value(left, left_value) &&
+                    ASRUtils::extract_value(right, right_value) ) {
+                    double mul_value = left_value * right_value;
+                    value = ASRUtils::EXPR(ASR::make_RealConstant_t(al, loc, mul_value, type));
+                }
                 return EXPR(ASR::make_RealBinOp_t(al, loc, left,
-                    ASR::binopType::Mul, right, type, nullptr));
+                    ASR::binopType::Mul, right, type, value));
             }
             case ASR::ttypeType::Complex: {
                 return EXPR(ASR::make_ComplexBinOp_t(al, loc, left,
@@ -436,7 +560,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -461,7 +585,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -486,7 +610,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -532,7 +656,7 @@ class ASRBuilder {
         return intrinsic_func(al, loc, scope, arg_types, return_type, new_args, overload_id);
     }
 
-    // Compare -----------------------------------------------------------------   
+    // Compare -----------------------------------------------------------------
     ASR::expr_t *Gt(ASR::expr_t *left, ASR::expr_t *right) {
         LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
         ASR::ttype_t *type = expr_type(left);
@@ -543,7 +667,7 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::Gt, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::Gt, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
@@ -551,7 +675,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -567,7 +691,7 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::Lt, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::Lt, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
@@ -575,7 +699,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -591,7 +715,7 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::GtE, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::GtE, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
@@ -599,7 +723,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -615,7 +739,7 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::LtE, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::LtE, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
@@ -623,7 +747,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -639,15 +763,18 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
                 return EXPR(ASR::make_LogicalCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
             }
+            case ASR::ttypeType::Complex: {
+                return EXPR(ASR::make_ComplexCompare_t(al, loc, left, ASR::cmpopType::Eq, right, logical, nullptr));
+            }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -663,7 +790,7 @@ class ASRBuilder {
             case ASR::ttypeType::Real: {
                 return EXPR(ASR::make_RealCompare_t(al, loc, left, ASR::cmpopType::NotEq, right, logical, nullptr));
             }
-            case ASR::ttypeType::Character: {
+            case ASR::ttypeType::String: {
                 return EXPR(ASR::make_StringCompare_t(al, loc, left, ASR::cmpopType::NotEq, right, logical, nullptr));
             }
             case ASR::ttypeType::Logical: {
@@ -671,7 +798,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    std::to_string(expr_type(left)->type) + " not yet supported");
+                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
                 return nullptr;
             }
         }
@@ -776,7 +903,8 @@ class ASRBuilder {
     }
 
     ASR::stmt_t *Assignment(ASR::expr_t *lhs, ASR::expr_t *rhs) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(lhs), expr_type(rhs)));
+        LCOMPILERS_ASSERT_MSG(check_equal_type(expr_type(lhs), expr_type(rhs)),
+            type_to_str_python(expr_type(lhs)) + ", " + type_to_str_python(expr_type(rhs)));
         return STMT(ASR::make_Assignment_t(al, loc, lhs, rhs, nullptr));
     }
 
@@ -969,8 +1097,7 @@ class ASRBuilder {
         // Used for debugging
         Vec<ASR::expr_t *> x_exprs;
         x_exprs.from_pointer_n_copy(al, &items[0], items.size());
-        return STMT(ASR::make_Print_t(al, loc, x_exprs.p, x_exprs.n,
-            nullptr, nullptr));
+        return STMT(ASRUtils::make_print_t_util(al, loc, x_exprs.p, x_exprs.n));
     }
 
     ASR::symbol_t* create_c_func(std::string c_func_name, SymbolTable* fn_symtab, ASR::ttype_t* return_type, int n_args, Vec<ASR::ttype_t*>& arg_types) {
