@@ -6,6 +6,8 @@
 #include <libasr/pass/intrinsic_function_registry.h>
 #include <libasr/pass/intrinsic_array_function_registry.h>
 
+#include <set>
+
 namespace LCompilers {
 
 namespace ASR {
@@ -51,6 +53,8 @@ private:
     std::set<std::pair<uint64_t, std::string>> const_assigned;
 
     bool symbol_visited;
+    bool _return_var_or_intent_out = false;
+    bool _processing_dims = false;
 
 public:
     VerifyVisitor(bool check_external, diag::Diagnostics &diagnostics) : check_external{check_external},
@@ -76,19 +80,19 @@ public:
                         // The symbol table was found and the symbol `sym` is in it
                         return true;
                     } else {
-                        diagnostics.message_label("ASR verify: The symbol table was found and the symbol in it shares the name, but is not equal to `sym`",
+                        diagnostics.message_label("The symbol table was found and the symbol in it shares the name, but is not equal to `sym`",
                         {sym->base.loc}, "failed here", diag::Level::Error, diag::Stage::ASRVerify);
                         return false;
                     }
                 } else {
-                    diagnostics.message_label("ASR verify: The symbol table was found, but the symbol `sym` is not in it",
+                    diagnostics.message_label("The symbol table was found, but the symbol `sym` is not in it",
                         {sym->base.loc}, "failed here", diag::Level::Error, diag::Stage::ASRVerify);
                     return false;
                 }
             }
             s = s->parent;
         }
-        diagnostics.message_label("ASR verify: The symbol table was not found in the scope of `symtab`.",
+        diagnostics.message_label("The symbol table was not found in the scope of `symtab`.",
                         {sym->base.loc}, "failed here", diag::Level::Error, diag::Stage::ASRVerify);
         return false;
     }
@@ -122,6 +126,20 @@ public:
         current_symtab = nullptr;
     }
 
+    void visit_Select(const Select_t& x) {
+        bool fall_through = false;
+        for( size_t i = 0; i < x.n_body; i++ ) {
+            if( ASR::is_a<ASR::CaseStmt_t>(*x.m_body[i]) ) {
+                ASR::CaseStmt_t* case_stmt_t = ASR::down_cast<ASR::CaseStmt_t>(x.m_body[i]);
+                fall_through = fall_through || case_stmt_t->m_fall_through;
+            }
+        }
+        require(fall_through == x.m_enable_fall_through,
+            "Select_t::m_enable_fall_through should be " +
+            std::to_string(x.m_enable_fall_through));
+        BaseWalkVisitor<VerifyVisitor>::visit_Select(x);
+    }
+
     // --------------------------------------------------------
     // symbol instances:
 
@@ -345,13 +363,17 @@ public:
         ASR::expr_t* target = x.m_target;
         if( ASR::is_a<ASR::Var_t>(*target) ) {
             ASR::Var_t* target_Var = ASR::down_cast<ASR::Var_t>(target);
+            bool is_target_const = false;
             ASR::ttype_t* target_type = nullptr;
-            if( ASR::is_a<ASR::Variable_t>(*target_Var->m_v) ||
-                (ASR::is_a<ASR::ExternalSymbol_t>(*target_Var->m_v) &&
-                 ASR::down_cast<ASR::ExternalSymbol_t>(target_Var->m_v)->m_external) ) {
-                target_type = ASRUtils::expr_type(target);
+            ASR::symbol_t* target_sym = ASRUtils::symbol_get_past_external(target_Var->m_v);
+            if( target_sym && ASR::is_a<ASR::Variable_t>(*target_sym) ) {
+                ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(target_sym);
+                require(var->m_intent != ASR::intentType::In, "Assignment target `"
+                    + std::string(var->m_name) + "` with intent `IN` not allowed");
+                target_type = var->m_type;
+                is_target_const = var->m_storage == ASR::storage_typeType::Parameter;
             }
-            if( target_type && ASR::is_a<ASR::Const_t>(*target_type) ) {
+            if( is_target_const ) {
                 std::string variable_name = ASRUtils::symbol_name(target_Var->m_v);
                 require(const_assigned.find(std::make_pair(current_symtab->counter,
                     variable_name)) == const_assigned.end(),
@@ -402,6 +424,10 @@ public:
     }
 
     void visit_Function(const Function_t &x) {
+        if (ASRUtils::get_FunctionType(&x)->m_abi == abiType::Interactive) {
+            require(x.n_body == 0,
+            "The Function::n_body should be 0 if abi set to Interactive");
+        }
         std::vector<std::string> function_dependencies_copy = function_dependencies;
         function_dependencies.clear();
         function_dependencies.reserve(x.n_dependencies);
@@ -426,6 +452,7 @@ public:
             LCOMPILERS_ASSERT(a.second);
             this->visit_symbol(*a.second);
         }
+        visit_ttype(*x.m_function_signature);
         for (size_t i=0; i<x.n_args; i++) {
             LCOMPILERS_ASSERT(x.m_args[i]);
             visit_expr(*x.m_args[i]);
@@ -485,15 +512,15 @@ public:
         SymbolTable *parent_symtab = current_symtab;
         current_symtab = x.m_symtab;
         require(x.m_name != nullptr,
-            "The StructType::m_name cannot be nullptr");
+            "The Struct::m_name cannot be nullptr");
         require(x.m_symtab != nullptr,
-            "The StructType::m_symtab cannot be nullptr");
+            "The Struct::m_symtab cannot be nullptr");
         require(x.m_symtab->parent == parent_symtab,
-            "The StructType::m_symtab->parent is not the right parent");
+            "The Struct::m_symtab->parent is not the right parent");
         require(x.m_symtab->asr_owner == (ASR::asr_t*)&x,
             "The X::m_symtab::asr_owner must point to X");
         require(id_symtab_map.find(x.m_symtab->counter) == id_symtab_map.end(),
-            "StructType::m_symtab->counter must be unique");
+            "Struct::m_symtab->counter must be unique");
         require(ASRUtils::symbol_symtab(down_cast<symbol_t>(current_symtab->asr_owner)) == current_symtab,
             "The asr_owner invariant failed");
         id_symtab_map[x.m_symtab->counter] = x.m_symtab;
@@ -502,26 +529,28 @@ public:
             this->visit_symbol(*a.second);
             if( ASR::is_a<ASR::ClassProcedure_t>(*a.second) ||
                 ASR::is_a<ASR::GenericProcedure_t>(*a.second) ||
-                ASR::is_a<ASR::StructType_t>(*a.second) ||
-                ASR::is_a<ASR::UnionType_t>(*a.second) ||
+                ASR::is_a<ASR::Struct_t>(*a.second) ||
+                ASR::is_a<ASR::Union_t>(*a.second) ||
                 ASR::is_a<ASR::ExternalSymbol_t>(*a.second) ||
                 ASR::is_a<ASR::CustomOperator_t>(*a.second) ) {
                 continue ;
             }
+            // TODO: Uncomment the following line
+            // ASR::ttype_t* var_type = ASRUtils::extract_type(ASRUtils::symbol_type(a.second));
             ASR::ttype_t* var_type = ASRUtils::type_get_past_pointer(ASRUtils::symbol_type(a.second));
             char* aggregate_type_name = nullptr;
             ASR::symbol_t* sym = nullptr;
-            if( ASR::is_a<ASR::Struct_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::Struct_t>(var_type)->m_derived_type;
+            if( ASR::is_a<ASR::StructType_t>(*var_type) ) {
+                sym = ASR::down_cast<ASR::StructType_t>(var_type)->m_derived_type;
                 aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::Enum_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::Enum_t>(var_type)->m_enum_type;
+            } else if( ASR::is_a<ASR::EnumType_t>(*var_type) ) {
+                sym = ASR::down_cast<ASR::EnumType_t>(var_type)->m_enum_type;
                 aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::Union_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::Union_t>(var_type)->m_union_type;
+            } else if( ASR::is_a<ASR::UnionType_t>(*var_type) ) {
+                sym = ASR::down_cast<ASR::UnionType_t>(var_type)->m_union_type;
                 aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::Class_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::Class_t>(var_type)->m_class_type;
+            } else if( ASR::is_a<ASR::ClassType_t>(*var_type) ) {
+                sym = ASR::down_cast<ASR::ClassType_t>(var_type)->m_class_type;
                 aggregate_type_name = ASRUtils::symbol_name(sym);
             }
             if( aggregate_type_name && ASRUtils::symbol_parent_symtab(sym) != current_symtab ) {
@@ -543,7 +572,7 @@ public:
         current_symtab = parent_symtab;
     }
 
-    void visit_StructType(const StructType_t& x) {
+    void visit_Struct(const Struct_t& x) {
         visit_UserDefinedType(x);
         if( !x.m_alignment ) {
             return ;
@@ -558,23 +587,23 @@ public:
                 " is not a positive power of 2.");
     }
 
-    void visit_EnumType(const EnumType_t& x) {
+    void visit_Enum(const Enum_t& x) {
         visit_UserDefinedType(x);
         require(x.m_type != nullptr,
-            "The common type of Enum cannot be nullptr. " +
+            "The common type of EnumType cannot be nullptr. " +
             std::string(x.m_name) + " doesn't seem to follow this rule.");
         ASR::ttype_t* common_type = x.m_type;
         std::map<int64_t, int64_t> value2count;
         for( auto itr: x.m_symtab->get_scope() ) {
             ASR::Variable_t* itr_var = ASR::down_cast<ASR::Variable_t>(itr.second);
             require(itr_var->m_symbolic_value != nullptr,
-                "All members of Enum must have their values to be set. " +
+                "All members of EnumType must have their values to be set. " +
                 std::string(itr_var->m_name) + " doesn't seem to follow this rule in "
-                + std::string(x.m_name) + " Enum.");
+                + std::string(x.m_name) + " EnumType.");
             require(ASRUtils::check_equal_type(itr_var->m_type, common_type),
-                "All members of Enum must the same type. " +
+                "All members of EnumType must the same type. " +
                 std::string(itr_var->m_name) + " doesn't seem to follow this rule in " +
-                std::string(x.m_name) + " Enum.");
+                std::string(x.m_name) + " EnumType.");
             ASR::expr_t* value = ASRUtils::expr_value(itr_var->m_symbolic_value);
             int64_t value_int64 = -1;
             ASRUtils::extract_value(value, value_int64);
@@ -608,10 +637,10 @@ public:
             is_enumtype_correct = !is_enum_integer;
         }
         require(is_enumtype_correct, "Properties of enum value members don't match correspond "
-                                     "to EnumType::m_enum_value_type");
+                                     "to Enum::m_enum_value_type");
     }
 
-    void visit_UnionType(const UnionType_t& x) {
+    void visit_Union(const Union_t& x) {
         visit_UserDefinedType(x);
     }
 
@@ -637,7 +666,7 @@ public:
             if (ASR::is_a<ASR::Module_t>(*asr_owner_sym)) {
                 is_module = true;
             }
-            if (ASR::is_a<ASR::StructType_t>(*asr_owner_sym)) {
+            if (ASR::is_a<ASR::Struct_t>(*asr_owner_sym)) {
                 is_struct = true;
             }
         }
@@ -646,8 +675,8 @@ public:
             // For now restrict this check only to variables which are present
             // inside symbols which have a body.
             require( (x.m_symbolic_value == nullptr && x.m_value == nullptr) ||
-                     (x.m_symbolic_value != nullptr && x.m_value != nullptr) ||
-                     (x.m_symbolic_value != nullptr && ASRUtils::is_value_constant(x.m_symbolic_value)),
+                    (x.m_symbolic_value != nullptr && x.m_value != nullptr) ||
+                    (x.m_symbolic_value != nullptr && ASRUtils::is_value_constant(x.m_symbolic_value)),
                     "Initialisation of " + std::string(x.m_name) +
                     " must reduce to a compile time constant.");
         }
@@ -656,7 +685,11 @@ public:
             visit_expr(*x.m_symbolic_value);
         if (x.m_value)
              visit_expr(*x.m_value);
+        _return_var_or_intent_out = x.m_intent == ASR::intentType::Out ||
+                                    x.m_intent == ASR::intentType::InOut ||
+                                    x.m_intent == ASR::intentType::ReturnVar;
         visit_ttype(*x.m_type);
+        _return_var_or_intent_out = false;
 
         verify_unique_dependencies(x.m_dependencies, x.n_dependencies,
                                    x.m_name, x.base.base.loc);
@@ -689,23 +722,28 @@ public:
             require(std::string(x.m_original_name) == std::string(orig_name),
                 "ExternalSymbol::m_original_name must match external->m_name");
             ASR::Module_t *m = ASRUtils::get_sym_module(x.m_external);
-            ASR::StructType_t* sm = nullptr;
-            ASR::EnumType_t* em = nullptr;
+            ASR::Struct_t* sm = nullptr;
+            ASR::Enum_t* em = nullptr;
+            ASR::Union_t* um = nullptr;
             ASR::Function_t* fm = nullptr;
             bool is_valid_owner = false;
             is_valid_owner = m != nullptr && ((ASR::symbol_t*) m == ASRUtils::get_asr_owner(x.m_external));
             std::string asr_owner_name = "";
             if( !is_valid_owner ) {
                 ASR::symbol_t* asr_owner_sym = ASRUtils::get_asr_owner(x.m_external);
-                is_valid_owner = (ASR::is_a<ASR::StructType_t>(*asr_owner_sym) ||
-                                  ASR::is_a<ASR::EnumType_t>(*asr_owner_sym) ||
-                                  ASR::is_a<ASR::Function_t>(*asr_owner_sym));
-                if( ASR::is_a<ASR::StructType_t>(*asr_owner_sym) ) {
-                    sm = ASR::down_cast<ASR::StructType_t>(asr_owner_sym);
+                is_valid_owner = (ASR::is_a<ASR::Struct_t>(*asr_owner_sym) ||
+                                  ASR::is_a<ASR::Enum_t>(*asr_owner_sym) ||
+                                  ASR::is_a<ASR::Function_t>(*asr_owner_sym) ||
+                                  ASR::is_a<ASR::Union_t>(*asr_owner_sym));
+                if( ASR::is_a<ASR::Struct_t>(*asr_owner_sym) ) {
+                    sm = ASR::down_cast<ASR::Struct_t>(asr_owner_sym);
                     asr_owner_name = sm->m_name;
-                } else if( ASR::is_a<ASR::EnumType_t>(*asr_owner_sym) ) {
-                    em = ASR::down_cast<ASR::EnumType_t>(asr_owner_sym);
+                } else if( ASR::is_a<ASR::Enum_t>(*asr_owner_sym) ) {
+                    em = ASR::down_cast<ASR::Enum_t>(asr_owner_sym);
                     asr_owner_name = em->m_name;
+                } else if( ASR::is_a<ASR::Union_t>(*asr_owner_sym) ) {
+                    um = ASR::down_cast<ASR::Union_t>(asr_owner_sym);
+                    asr_owner_name = um->m_name;
                 } else if( ASR::is_a<ASR::Function_t>(*asr_owner_sym) ) {
                     fm = ASR::down_cast<ASR::Function_t>(asr_owner_sym);
                     asr_owner_name = fm->m_name;
@@ -734,6 +772,8 @@ public:
                 s = em->m_symtab->resolve_symbol(std::string(x.m_original_name));
             } else if( fm ) {
                 s = fm->m_symtab->resolve_symbol(std::string(x.m_original_name));
+            } else if( um ) {
+                s = um->m_symtab->resolve_symbol(std::string(x.m_original_name));
             }
             require(s != nullptr,
                 "ExternalSymbol::m_original_name ('"
@@ -759,9 +799,9 @@ public:
             s = ASRUtils::symbol_get_past_external(x.m_v);
         }
         require(is_a<Variable_t>(*s) || is_a<Function_t>(*s)
-                || is_a<ASR::EnumType_t>(*s) || is_a<ASR::ExternalSymbol_t>(*s),
+                || is_a<ASR::Enum_t>(*s) || is_a<ASR::ExternalSymbol_t>(*s),
             "Var_t::m_v " + x_mv_name + " does not point to a Variable_t, " \
-            "Function_t, or EnumType_t (possibly behind ExternalSymbol_t)");
+            "Function_t, or Enum_t (possibly behind ExternalSymbol_t)");
         require(symtab_in_scope(current_symtab, x.m_v),
             "Var::m_v `" + x_mv_name + "` cannot point outside of its symbol table");
         variable_dependencies.push_back(x_mv_name);
@@ -802,7 +842,7 @@ public:
             check_var_external(*x.m_v);
             int n_dims = ASRUtils::extract_n_dims_from_ttype(
                     ASRUtils::expr_type(x.m_v));
-            if (ASR::is_a<ASR::Character_t>(*x.m_type) && n_dims == 0) {
+            if (ASR::is_a<ASR::String_t>(*x.m_type) && n_dims == 0) {
                 // TODO: This seems like a bug, we should not use ArrayItem with
                 // strings but StringItem. For now we ignore it, but we should
                 // fix it
@@ -814,10 +854,35 @@ public:
     }
 
     void visit_ArrayItem(const ArrayItem_t &x) {
+        if (ASRUtils::use_experimental_simplifier) {
+            if( check_external ) {
+                if( ASRUtils::is_array_indexed_with_array_indices(x.m_args, x.n_args) ) {
+                    require(ASRUtils::is_array(x.m_type),
+                        "ArrayItem::m_type with array indices must be an array.")
+                } else {
+                    require(!ASRUtils::is_array(x.m_type),
+                        "ArrayItem::m_type cannot be array.")
+                }
+            }
+        }
         handle_ArrayItemSection(x);
     }
 
+    void visit_ArraySize(const ArraySize_t& x) {
+        if (ASRUtils::use_experimental_simplifier) {
+            if (check_external) {
+                require(ASRUtils::is_array(ASRUtils::expr_type(x.m_v)),
+                    "ArraySize::m_v must be an array");
+            }
+        }
+        BaseWalkVisitor<VerifyVisitor>::visit_ArraySize(x);
+    }
+
     void visit_ArraySection(const ArraySection_t &x) {
+        require(
+            ASR::is_a<ASR::Array_t>(*x.m_type),
+            "ArrayItemSection::m_type can only be an Array"
+        );
         handle_ArrayItemSection(x);
     }
 
@@ -858,11 +923,13 @@ public:
             require(x.m_new != x.m_old, "ArrayPhysicalCast is redundant, "
                 "the old physical type and new physical type must be different.");
         }
-        require(x.m_new == ASRUtils::extract_physical_type(x.m_type),
-            "Destination physical type conflicts with the physical type of target");
-        require(x.m_old == ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg)),
-            "Old physical type conflicts with the physical type of argument " + std::to_string(x.m_old)
-            + " " + std::to_string(ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg))));
+        if(check_external){
+            require(x.m_new == ASRUtils::extract_physical_type(x.m_type),
+                "Destination physical type conflicts with the physical type of target");
+            require(x.m_old == ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg)),
+                "Old physical type conflicts with the physical type of argument " + std::to_string(x.m_old)
+                + " " + std::to_string(ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg))));
+        }
     }
 
     void visit_SubroutineCall(const SubroutineCall_t &x) {
@@ -872,7 +939,7 @@ public:
             ASR::symbol_t *s = ASRUtils::symbol_get_past_external(x.m_name);
             if (ASR::is_a<ASR::Variable_t>(*s)) {
                 ASR::Variable_t *v = ASR::down_cast<ASR::Variable_t>(s);
-                require(v->m_type_declaration && ASR::is_a<ASR::Function_t>(*v->m_type_declaration),
+                require(v->m_type_declaration && ASR::is_a<ASR::Function_t>(*ASRUtils::symbol_get_past_external(v->m_type_declaration)),
                     "SubroutineCall::m_name '" + std::string(symbol_name(x.m_name)) + "' is a Variable, but does not point to Function");
                 require(ASR::is_a<ASR::FunctionType_t>(*v->m_type),
                     "SubroutineCall::m_name '" + std::string(symbol_name(x.m_name)) + "' is a Variable, but the type is not FunctionType");
@@ -912,6 +979,12 @@ public:
         verify_args(x);
     }
 
+    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
+        require(symtab_in_scope(current_symtab, x.m_m),
+            "AssociateBlockCall::m_name '" + std::string(symbol_name(x.m_m)) +
+                "' cannot point outside of its symbol table");
+    }
+
     SymbolTable *get_dt_symtab(ASR::symbol_t *dt) {
         LCOMPILERS_ASSERT(dt)
         SymbolTable *symtab = ASRUtils::symbol_symtab(ASRUtils::symbol_get_past_external(dt));
@@ -925,17 +998,17 @@ public:
         ASR::ttype_t *t2 = ASRUtils::type_get_past_pointer(ASRUtils::expr_type(dt));
         ASR::symbol_t *type_sym=nullptr;
         switch (t2->type) {
-            case (ASR::ttypeType::Struct): {
-                type_sym = ASR::down_cast<ASR::Struct_t>(t2)->m_derived_type;
+            case (ASR::ttypeType::StructType): {
+                type_sym = ASR::down_cast<ASR::StructType_t>(t2)->m_derived_type;
                 break;
             }
-            case (ASR::ttypeType::Class): {
-                type_sym = ASR::down_cast<ASR::Class_t>(t2)->m_class_type;
+            case (ASR::ttypeType::ClassType): {
+                type_sym = ASR::down_cast<ASR::ClassType_t>(t2)->m_class_type;
                 break;
             }
             default :
                 require_with_loc(false,
-                    "m_dt::m_v::m_type must point to a type with a symbol table (Struct or Class)",
+                    "m_dt::m_v::m_type must point to a type with a symbol table (StructType or ClassType)",
                     dt->base.loc);
         }
         return get_dt_symtab(type_sym);
@@ -944,15 +1017,15 @@ public:
     ASR::symbol_t *get_parent_type_dt(ASR::symbol_t *dt) {
         ASR::symbol_t *parent = nullptr;
         switch (dt->type) {
-            case (ASR::symbolType::StructType): {
+            case (ASR::symbolType::Struct): {
                 dt = ASRUtils::symbol_get_past_external(dt);
-                ASR::StructType_t* der_type = ASR::down_cast<ASR::StructType_t>(dt);
+                ASR::Struct_t* der_type = ASR::down_cast<ASR::Struct_t>(dt);
                 parent = der_type->m_parent;
                 break;
             }
             default :
                 require_with_loc(false,
-                    "m_dt::m_v::m_type must point to a Struct type",
+                    "m_dt::m_v::m_type must point to a StructType type",
                     dt->base.loc);
         }
         return parent;
@@ -963,25 +1036,25 @@ public:
         ASR::symbol_t *type_sym=nullptr;
         ASR::symbol_t *parent = nullptr;
         switch (t2->type) {
-            case (ASR::ttypeType::Struct): {
-                type_sym = ASR::down_cast<ASR::Struct_t>(t2)->m_derived_type;
+            case (ASR::ttypeType::StructType): {
+                type_sym = ASR::down_cast<ASR::StructType_t>(t2)->m_derived_type;
                 type_sym = ASRUtils::symbol_get_past_external(type_sym);
-                ASR::StructType_t* der_type = ASR::down_cast<ASR::StructType_t>(type_sym);
+                ASR::Struct_t* der_type = ASR::down_cast<ASR::Struct_t>(type_sym);
                 parent = der_type->m_parent;
                 break;
             }
-            case (ASR::ttypeType::Class): {
-                type_sym = ASR::down_cast<ASR::Class_t>(t2)->m_class_type;
+            case (ASR::ttypeType::ClassType): {
+                type_sym = ASR::down_cast<ASR::ClassType_t>(t2)->m_class_type;
                 type_sym = ASRUtils::symbol_get_past_external(type_sym);
-                if( type_sym->type == ASR::symbolType::StructType ) {
-                    ASR::StructType_t* der_type = ASR::down_cast<ASR::StructType_t>(type_sym);
+                if( type_sym->type == ASR::symbolType::Struct ) {
+                    ASR::Struct_t* der_type = ASR::down_cast<ASR::Struct_t>(type_sym);
                     parent = der_type->m_parent;
                 }
                 break;
             }
             default :
                 require_with_loc(false,
-                    "m_dt::m_v::m_type must point to a Struct type",
+                    "m_dt::m_v::m_type must point to a StructType type",
                     dt->base.loc);
         }
         return parent;
@@ -1007,16 +1080,16 @@ public:
         }
     }
 
-    void visit_IntrinsicScalarFunction(const ASR::IntrinsicScalarFunction_t& x) {
+    void visit_IntrinsicElementalFunction(const ASR::IntrinsicElementalFunction_t& x) {
         if( !check_external ) {
-            BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicScalarFunction(x);
+            BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicElementalFunction(x);
             return ;
         }
-        ASRUtils::verify_function verify_ = ASRUtils::IntrinsicScalarFunctionRegistry
+        ASRUtils::verify_function verify_ = ASRUtils::IntrinsicElementalFunctionRegistry
             ::get_verify_function(x.m_intrinsic_id);
         LCOMPILERS_ASSERT(verify_ != nullptr);
         verify_(x, diagnostics);
-        BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicScalarFunction(x);
+        BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicElementalFunction(x);
     }
 
     void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t& x) {
@@ -1052,6 +1125,11 @@ public:
                 function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
             }
         }
+        if (_return_var_or_intent_out  && _processing_dims &&
+            temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
+            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name)) {
+            function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+        }
 
         if( ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) ) {
             ASR::ExternalSymbol_t* x_m_name = ASR::down_cast<ASR::ExternalSymbol_t>(x.m_name);
@@ -1085,36 +1163,53 @@ public:
         visit_ttype(*x.m_type);
     }
 
-    void visit_Struct(const Struct_t &x) {
+    void visit_StructType(const StructType_t &x) {
         std::string symbol_owner = "global scope";
         if( ASRUtils::get_asr_owner(x.m_derived_type) ) {
             symbol_owner = ASRUtils::symbol_name(ASRUtils::get_asr_owner(x.m_derived_type));
         }
         require(symtab_in_scope(current_symtab, x.m_derived_type),
-            "Struct::m_derived_type '" +
+            "StructType::m_derived_type '" +
             std::string(ASRUtils::symbol_name(x.m_derived_type)) +
             "' cannot point outside of its symbol table, owner: " +
             symbol_owner);
     }
 
+    void visit_ArrayConstructor(const ArrayConstructor_t& x) {
+        require(ASRUtils::is_array(x.m_type),
+            "Type of ArrayConstructor must be an array");
+        BaseWalkVisitor<VerifyVisitor>::visit_ArrayConstructor(x);
+    }
+
     void visit_ArrayConstant(const ArrayConstant_t& x) {
         require(ASRUtils::is_array(x.m_type),
             "Type of ArrayConstant must be an array");
-        BaseWalkVisitor<VerifyVisitor>::visit_ArrayConstant(x);
+
+        int64_t n_data = ASRUtils::get_fixed_size_of_array(x.m_type) * ASRUtils::extract_kind_from_ttype_t(x.m_type);
+        if (ASRUtils::is_character(*x.m_type)) {
+            ASR::ttype_t* t = ASRUtils::type_get_past_array(x.m_type);
+            n_data = ASRUtils::get_fixed_size_of_array(x.m_type) * ASR::down_cast<ASR::String_t>(t)->m_len;
+        }
+        require(n_data == x.m_n_data, "ArrayConstant::m_n_data must match the byte size of the array");
+        visit_ttype(*x.m_type);
     }
 
     void visit_dimension(const dimension_t &x) {
         if (x.m_start) {
-            require_with_loc(ASRUtils::is_integer(
-                *ASRUtils::type_get_past_const(ASRUtils::expr_type(x.m_start))),
-                "Start dimension must be a signed integer", x.loc);
+            if(check_external){
+                require_with_loc(ASRUtils::is_integer(
+                    *ASRUtils::expr_type(x.m_start)),
+                    "Start dimension must be a signed integer", x.loc);
+            }
             visit_expr(*x.m_start);
         }
 
         if (x.m_length) {
-            require_with_loc(ASRUtils::is_integer(
-                *ASRUtils::type_get_past_const(ASRUtils::expr_type(x.m_length))),
-                "Length dimension must be a signed integer", x.loc);
+            if(check_external){
+                require_with_loc(ASRUtils::is_integer(
+                    *ASRUtils::expr_type(x.m_length)),
+                    "Length dimension must be a signed integer", x.loc);
+            }
             visit_expr(*x.m_length);
         }
     }
@@ -1125,9 +1220,11 @@ public:
         visit_ttype(*x.m_type);
         require(x.n_dims != 0, "Array type cannot have 0 dimensions.")
         require(!ASR::is_a<ASR::Array_t>(*x.m_type), "Array type cannot be nested.")
+        _processing_dims = true;
         for (size_t i = 0; i < x.n_dims; i++) {
             visit_dimension(x.m_dims[i]);
         }
+        _processing_dims = false;
     }
 
     void visit_Pointer(const Pointer_t &x) {
@@ -1147,19 +1244,41 @@ public:
     void visit_Allocatable(const Allocatable_t &x) {
         require(!ASR::is_a<ASR::Pointer_t>(*x.m_type),
             "Allocatable type conflicts with Pointer type");
+        if (ASRUtils::use_experimental_simplifier) {
+            ASR::dimension_t* m_dims = nullptr;
+            size_t n_dims = ASRUtils::extract_dimensions_from_ttype(x.m_type, m_dims);
+            for( size_t i = 0; i < n_dims; i++ ) {
+                require(m_dims[i].m_length == nullptr,
+                    "Length of allocatable should be deferred (empty).");
+            }
+        }
         visit_ttype(*x.m_type);
     }
 
     void visit_Allocate(const Allocate_t &x) {
-        for( size_t i = 0; i < x.n_args; i++ ) {
-            require(ASR::is_a<ASR::Allocatable_t>(*ASRUtils::expr_type(x.m_args[i].m_a)) ||
-                    ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x.m_args[i].m_a)),
-                "Allocate should only be called with  Allocatable or Pointer type inputs, found " +
-                std::string(ASRUtils::get_type_code(ASRUtils::expr_type(x.m_args[i].m_a))));
+        if(check_external){
+            for( size_t i = 0; i < x.n_args; i++ ) {
+                require(ASR::is_a<ASR::Allocatable_t>(*ASRUtils::expr_type(x.m_args[i].m_a)) ||
+                        ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x.m_args[i].m_a)),
+                    "Allocate should only be called with  Allocatable or Pointer type inputs, found " +
+                    std::string(ASRUtils::get_type_code(ASRUtils::expr_type(x.m_args[i].m_a))));
+            }
         }
         BaseWalkVisitor<VerifyVisitor>::visit_Allocate(x);
     }
 
+    void visit_DoConcurrentLoop(const DoConcurrentLoop_t &x) {
+        for ( size_t i = 0; i < x.n_local; i++ ) {
+            require(ASR::is_a<ASR::Var_t>(*x.m_local[i]),
+                "DoConcurrentLoop::m_local must be a Var");
+        }
+        for ( size_t i = 0; i < x.n_shared; i++ ) {
+            require(ASR::is_a<ASR::Var_t>(*x.m_shared[i]),
+                "DoConcurrentLoop::m_shared must be a Var");
+        }
+        BaseWalkVisitor<VerifyVisitor>::visit_DoConcurrentLoop(x);
+    }
+
 };
 
 
