@@ -320,7 +320,7 @@ public:
         for (auto &item : x.m_symtab->get_scope()) {
             if (ASR::is_a<ASR::Struct_t>(*item.second) ||
                     ASR::is_a<ASR::Enum_t>(*item.second) ||
-                    ASR::is_a<ASR::UnionType_t>(*item.second)) {
+                    ASR::is_a<ASR::Union_t>(*item.second)) {
                 std::vector<std::string> struct_deps_vec;
                 std::pair<char**, size_t> struct_deps_ptr = ASRUtils::symbol_dependencies(item.second);
                 for( size_t i = 0; i < struct_deps_ptr.second; i++ ) {
@@ -378,6 +378,9 @@ public:
 
     void visit_Module(const ASR::Module_t &x) {
         std::string r;
+        if (strcmp(x.m_name,"lfortran_intrinsic_iso_c_binding")==0 && x.m_intrinsic) {
+            return;
+        }
         r = "module";
         r += " ";
         r.append(x.m_name);
@@ -402,7 +405,7 @@ public:
         for (auto &item : x.m_symtab->get_scope()) {
             if (ASR::is_a<ASR::Struct_t>(*item.second) ||
                     ASR::is_a<ASR::Enum_t>(*item.second) ||
-                    ASR::is_a<ASR::UnionType_t>(*item.second)) {
+                    ASR::is_a<ASR::Union_t>(*item.second)) {
                 std::vector<std::string> struct_deps_vec;
                 std::pair<char**, size_t> struct_deps_ptr = ASRUtils::symbol_dependencies(item.second);
                 for( size_t i = 0; i < struct_deps_ptr.second; i++ ) {
@@ -616,7 +619,17 @@ public:
     void visit_ExternalSymbol(const ASR::ExternalSymbol_t &x) {
         ASR::symbol_t *sym = down_cast<ASR::symbol_t>(
             ASRUtils::symbol_parent_symtab(x.m_external)->asr_owner);
-        if (!is_a<ASR::Struct_t>(*sym)) {
+        if (strcmp(x.m_module_name,"lfortran_intrinsic_iso_c_binding")==0 &&
+            sym && ASR::is_a<ASR::Module_t>(*sym) && ASR::down_cast<ASR::Module_t>(sym)->m_intrinsic) {
+            src = indent;
+            src += "use ";
+            src += "iso_c_binding";
+            src += ", only: ";
+            src.append(x.m_original_name);
+            src += "\n";
+            return;
+        }
+        if (!is_a<ASR::Struct_t>(*sym) && !is_a<ASR::Enum_t>(*sym)) {
             src = indent;
             src += "use ";
             src.append(x.m_module_name);
@@ -648,9 +661,27 @@ public:
         src = r;
     }
 
-    // void visit_Enum(const ASR::Enum_t &x) {}
+    void visit_Enum(const ASR::Enum_t &x) {
+        std::string r = indent;
+        r += "enum, bind(c)\n";
+        inc_indent();
+        for (auto it: x.m_symtab->get_scope()) {
+            ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(it.second);
+            r += indent;
+            r += "enumerator :: ";
+            r.append(var->m_name);
+            r += " = ";
+            visit_expr(*var->m_value);
+            r += src;
+            r += "\n";
+        }
+        dec_indent();
+        r += indent;
+        r += "end enum\n";
+        src = r;
+    }
 
-    // void visit_UnionType(const ASR::UnionType_t &x) {}
+    // void visit_Union(const ASR::Union_t &x) {}
 
     void visit_Variable(const ASR::Variable_t &x) {
         std::string r = indent;
@@ -690,6 +721,9 @@ public:
         if (x.m_value_attr) {
             r += ", value";
         }
+        if (x.m_target_attr) {
+            r += ", target";
+        }
         r += " :: ";
         r.append(x.m_name);
         if (x.m_symbolic_value && x.m_value && ASR::is_a<ASR::StringChr_t>(*x.m_symbolic_value) && ASR::is_a<ASR::StringConstant_t>(*x.m_value)) {
@@ -817,7 +851,38 @@ public:
         src = r;
     }
 
-    // void visit_DoConcurrentLoop(const ASR::DoConcurrentLoop_t &x) {}
+    void visit_DoConcurrentLoop(const ASR::DoConcurrentLoop_t &x) {
+        std::string r = indent;
+
+        r += "do concurrent";
+        r += " ( ";
+        for (size_t i = 0; i < x.n_head; i++) {
+            visit_expr(*x.m_head[i].m_v);
+            r += src;
+            r += " = ";
+            visit_expr(*x.m_head[i].m_start);
+            r += src;
+            r += ": ";
+            visit_expr(*x.m_head[i].m_end);
+            r += src;
+            if (x.m_head[i].m_increment) {
+                r += ":";
+                visit_expr(*x.m_head[i].m_increment);
+                r += src;
+            }
+            if ( i < x.n_head - 1 ) {
+                r+=", ";
+            }
+        }
+        r+=" )";
+        handle_line_truncation(r, 2);
+        r += "\n";
+        visit_body(x, r);
+        r += indent;
+        r += "end do";
+        r += "\n";
+        src = r;
+    }
 
     void visit_DoLoop(const ASR::DoLoop_t &x) {
         std::string r = indent;
@@ -1209,7 +1274,21 @@ public:
         src = r;
     }
 
-    // void visit_Nullify(const ASR::Nullify_t &x) {}
+    void visit_Nullify(const ASR::Nullify_t &x) {
+        std::string r = indent;
+        r += "nullify (";
+        for (int i = 0; i < static_cast<int>(x.n_vars); i++) {
+            visit_expr(*x.m_vars[i]);
+            r += src;
+            if(i != static_cast<int>(x.n_vars-1)) {
+                r += ", ";
+            }
+        }
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
+        src = r;
+    }
 
     // void visit_Flush(const ASR::Flush_t &x) {}
 
@@ -1295,8 +1374,9 @@ public:
     }
 
     void visit_IntrinsicElementalFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicElementalFunction_t &x) {
+        src = "";
         out += func_name;
-        visit_expr(*x.m_args[0]);
+        if (x.n_args > 0) visit_expr(*x.m_args[0]);
         out += "(" + src;
         for (size_t i = 1; i < x.n_args; i++) {
             out += ", ";
@@ -1321,6 +1401,9 @@ public:
         else if(intrinsic_func_name == "Mergebits") intrinsic_func_name = "merge_bits";
         else if(intrinsic_func_name == "StringLenTrim") intrinsic_func_name = "len_trim";
         else if(intrinsic_func_name == "StringTrim") intrinsic_func_name = "trim";
+        else if(intrinsic_func_name == "MoveAlloc") intrinsic_func_name = "move_alloc";
+        else if(intrinsic_func_name == "CompilerOptions") intrinsic_func_name = "compiler_options";
+        else if(intrinsic_func_name == "CompilerVersion") intrinsic_func_name = "compiler_version";
         visit_IntrinsicElementalFunction_helper(out, intrinsic_func_name, x);
     }
 
@@ -1386,13 +1469,31 @@ public:
 
     // void visit_EnumConstructor(const ASR::EnumConstructor_t &x) {}
 
-    // void visit_UnionTypeConstructor(const ASR::UnionTypeConstructor_t &x) {}
+    // void visit_UnionConstructor(const ASR::UnionConstructor_t &x) {}
 
     void visit_ImpliedDoLoop(const ASR::ImpliedDoLoop_t &x) {
-        if (x.m_value) {
-            visit_expr(*x.m_value);
-            return;
+        std::string r = "("; 
+        for (size_t i = 0; i < x.n_values; i++) {
+            visit_expr(*x.m_values[i]);
+            r += src;
+            if (i != x.n_values - 1) r += ", "; 
         }
+        r += ", ";
+        visit_expr(*x.m_var);   
+        r += src;
+        r += " = ";
+        visit_expr(*x.m_start);
+        r += src;
+        r += ", ";
+        visit_expr(*x.m_end); 
+        r += src;
+        if (x.m_increment) {   
+            r += ", ";
+            visit_expr(*x.m_increment);
+            r += src;
+            }
+        r += ")"; 
+        src = r;  
         return;
     }
 
@@ -1543,7 +1644,11 @@ public:
 
     void visit_StringConstant(const ASR::StringConstant_t &x) {
         src = "\"";
-        src.append(x.m_s);
+        if(std::strcmp(x.m_s, "\n") == 0) {
+            src.append("\\n");
+        } else {
+            src.append(x.m_s);
+        }
         src += "\"";
         last_expr_precedence = Precedence::Ext;
     }
@@ -1832,7 +1937,10 @@ public:
 
     // void visit_CLoc(const ASR::CLoc_t &x) {}
 
-    // void visit_PointerToCPtr(const ASR::PointerToCPtr_t &x) {}
+    void visit_PointerToCPtr(const ASR::PointerToCPtr_t &x) {
+        visit_expr(*x.m_arg);
+        src = "c_loc(" + src + ")";
+    }
 
     // void visit_GetPointer(const ASR::GetPointer_t &x) {}
 
