@@ -1,8 +1,6 @@
 #ifndef LFORTRAN_LLVM_UTILS_H
 #define LFORTRAN_LLVM_UTILS_H
 
-#include <memory>
-
 #include <llvm/IR/Value.h>
 #include <llvm/IR/Module.h>
 #include <llvm/IR/IRBuilder.h>
@@ -327,10 +325,10 @@ namespace LCompilers {
 
             llvm::Type* getStructType(ASR::ttype_t* _type, llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getUnionType(ASR::UnionType_t* union_type,
+            llvm::Type* getUnion(ASR::Union_t* union_type,
                 llvm::Module* module, bool is_pointer=false);
 
-            llvm::Type* getUnionType(ASR::ttype_t* _type,
+            llvm::Type* getUnion(ASR::ttype_t* _type,
                 llvm::Module* module, bool is_pointer=false);
 
             llvm::Type* getClassType(ASR::Class_t* der_type, bool is_pointer=false);
@@ -380,6 +378,8 @@ namespace LCompilers {
                 ASR::ttype_t* asr_type, llvm::Module* module,
                 std::map<std::string, std::map<std::string, int>>& name2memidx);
 
+            llvm::Value* convert_kind(llvm::Value* val, llvm::Type* target_type);
+
 
             // Note: `llvm_utils->create_if_else` and `create_loop` are optional APIs
             // that do not have to be used. Many times, for more complicated
