@@ -125,6 +125,14 @@ def _compare_eq_dict(
         )
     return explanation
 
+def test_for_duplicates(test_data):
+    tests = test_data["test"]
+    filenames = [t["filename"] for t in tests]
+    if len(set(filenames)) != len(filenames):
+        print("There are duplicate test filenames:")
+        duplicates = [item for item in set(filenames) if filenames.count(item) > 1]
+        print(duplicates)
+        sys.exit(1)
 
 def fixdir(s: bytes) -> bytes:
     local_dir = os.getcwd()
@@ -314,7 +322,12 @@ def run_test(testname, basename, cmd, infile, update_reference=False,
         raise FileNotFoundError(
             f"The output json file '{jo}' for {testname} does not exist")
 
-    do = json.load(open(jo))
+    try:
+        do = json.load(open(jo))
+    except json.decoder.JSONDecodeError:
+        print("JSON failed to be decoded")
+        print(f"Filename: {jo}")
+        raise
     if update_reference:
         do_update_reference(jo, jr, do)
         return
@@ -388,8 +401,6 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
                         help="Skip LLVM tests")
     parser.add_argument("--skip-run-with-dbg", action="store_true",
                         help="Skip runtime tests with debugging information enabled")
-    parser.add_argument("--skip-cpptranslate", action="store_true",
-                        help="Skip tests for ast_openmp that depend on cpptranslate")
     parser.add_argument("-s", "--sequential", action="store_true",
                         help="Run all tests sequentially")
     parser.add_argument("--no-color", action="store_true",
@@ -411,7 +422,6 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
     verbose = args.verbose
     no_llvm = args.no_llvm
     skip_run_with_dbg = args.skip_run_with_dbg
-    skip_cpptranslate = args.skip_cpptranslate
     global no_color
     no_color = args.no_color
 
@@ -420,6 +430,7 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
         os.environ["PATH"] = os.path.join(SRC_DIR, "bin") \
             + os.pathsep + os.environ["PATH"]
     test_data = toml.load(open(os.path.join(ROOT_DIR, "tests", "tests.toml")))
+    test_for_duplicates(test_data)
     filtered_tests = test_data["test"]
     if specific_tests:
         filtered_tests = [test for test in filtered_tests if any(
@@ -445,7 +456,6 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
                 verbose=verbose,
                 no_llvm=no_llvm,
                 skip_run_with_dbg=True,
-                skip_cpptranslate=True,
                 no_color=True)
     filtered_tests = [test for test in filtered_tests if 'extrafiles' not in test]
 
@@ -459,7 +469,6 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
                         verbose=verbose,
                         no_llvm=no_llvm,
                         skip_run_with_dbg=skip_run_with_dbg,
-                        skip_cpptranslate=skip_cpptranslate,
                         no_color=no_color)
     # run in parallel
     else:
@@ -472,7 +481,6 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
             verbose=verbose,
             no_llvm=no_llvm,
             skip_run_with_dbg=skip_run_with_dbg,
-            skip_cpptranslate=skip_cpptranslate,
             no_color=no_color)
         with ThreadPoolExecutor() as ex:
             futures = ex.map(single_tester_partial_args, filtered_tests)
