@@ -29,7 +29,7 @@ intrinsic_funcs_args = {
     "Mod": [
         {
             "args": [("int", "int"), ("real", "real")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": "dynamic"
         },
     ],
     "Trailz": [
@@ -68,6 +68,12 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 3
         },
     ],
+    "MoveAlloc": [
+        {
+            "args": [("any", "any")],
+            "ret_type_arg_idx": 0
+        },
+    ],
     "Leadz": [
         {
             "args": [("int",)],
@@ -83,7 +89,8 @@ intrinsic_funcs_args = {
     "Hypot": [
         {
             "args": [("real", "real")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg" : 2
         }
     ],
     "Trunc": [
@@ -128,6 +135,12 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0
         },
     ],
+    "ErfcScaled": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        },
+    ],
     "Atan2": [
         {
             "args": [("real", "real")],
@@ -170,6 +183,13 @@ intrinsic_funcs_args = {
             "return": "int32"
         }
     ],
+    "Logical": [
+        {
+            "args": [("bool", )],
+            "ret_type_arg_idx": 0,
+            "kind_arg": True
+        }
+    ],
     "Digits": [
         {
             "args": [("int",), ("real",)],
@@ -233,7 +253,8 @@ intrinsic_funcs_args = {
     "Sign": [
         {
             "args": [("int", "int"), ("real", "real")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg": 2
         },
     ],
     "Radix": [
@@ -242,6 +263,25 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
+    "IsContiguous": [
+        {
+            "args": [("any",)],
+            "return": "logical"
+        },
+    ],
+    "OutOfRange": [
+        {
+            "args": [("int", "real", "bool"), ("real", "real", "bool"), ("int", "int", "bool"), ("real", "int", "bool")],
+            "return": "logical"
+        },
+    ],
+    "StorageSize": [
+        {
+            "args": [("any",)],
+            "return": "int32",
+            "kind_arg": True
+        },
+    ],
     "Nearest": [
         {
             "args": [("real", "real")],
@@ -251,13 +291,13 @@ intrinsic_funcs_args = {
     "Adjustl": [
         {
             "args": [("char",)],
-            "return": "character(-1)"
+            "ret_type_arg_idx": 0
         }
     ],
     "Adjustr": [
         {
             "args": [("char",)],
-            "return": "character(-1)"
+            "ret_type_arg_idx": 0
         }
     ],
     "Aint": [
@@ -273,6 +313,12 @@ intrinsic_funcs_args = {
             "return": "logical",
         }
     ],
+    "SameTypeAs": [
+        {
+            "args": [("any", "any")],
+            "return": "logical"
+        }
+    ],
     "Nint": [
         {
             "args": [("real",)],
@@ -280,6 +326,12 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Idnint": [
+        {
+            "args": [("real",)],
+            "return": "int32"
+        }
+    ],
     "Anint": [
         {
             "args": [("real",)],
@@ -448,7 +500,7 @@ intrinsic_funcs_args = {
     "Sngl": [
         {
             "args": [("real",)],
-            "return": "real32"
+            "return": "real32",
         }
     ],
     "SignFromValue": [
@@ -532,19 +584,40 @@ intrinsic_funcs_args = {
     "Iand": [
         {
             "args": [("int", "int")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg": 2
+        },
+    ],
+    "And": [
+        {
+            "args": [("int", "int"),("bool","bool")],
+            "ret_type_arg_idx": 0,
         },
     ],
     "Ior": [
         {
             "args": [("int", "int")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg": 2
+        },
+    ],
+    "Or": [
+        {
+            "args": [("int", "int"), ("bool", "bool")],
+            "ret_type_arg_idx": 0,
         },
     ],
     "Ieor": [
         {
             "args": [("int", "int")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg": 2
+        },
+    ],
+    "Xor": [
+        {
+            "args": [("int", "int"), ("bool", "bool")],
+            "ret_type_arg_idx": 0,
         },
     ],
     "Ibclr": [
@@ -697,12 +770,13 @@ intrinsic_funcs_args = {
     "Mergebits": [
         {
             "args": [("int", "int", "int")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "same_kind_arg": 3
         }
-    ], 
+    ],
     "Ishftc": [
         {
-            "args": [("int", "int")],
+            "args": [("int", "int", "int")],
             "ret_type_arg_idx": 0
         },
     ],
@@ -720,6 +794,13 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Achar": [
+        {
+            "args": [("int",)],
+            "return": "character(1)",
+            "kind_arg": True
+        }
+    ],
     "Exponent": [
         {
             "args": [("real",)],
@@ -747,28 +828,64 @@ intrinsic_funcs_args = {
     "Dshiftl": [
        {
            "args": [("int", "int", "int",)],
-           "ret_type_arg_idx": 0
+           "ret_type_arg_idx": 0,
+           "same_kind_arg": 2
        },
-   ],
-   "Popcnt": [
-       {
-           "args": [("int",)],
-           "return": "int32",
-       },
-   ],
-   "Poppar": [
-       {
-           "args": [("int",)],
-           "return": "int32",
-       },
-   ],
-
+    ],
+    "Dshiftr": [
+        {
+            "args": [("int", "int", "int",)],
+            "ret_type_arg_idx": 0
+        },
+    ],
+    "Popcnt": [
+        {
+            "args": [("int",)],
+            "return": "int32",
+        },
+    ],
+    "Poppar": [
+        {
+            "args": [("int",)],
+            "return": "int32",
+        },
+    ],
+    "Real": [
+        {
+            "args": [("int",), ("real",), ("complex",)],
+            "return": "real32",
+            "kind_arg": True, 
+            "real_32_except_complex": True
+        },   
+    ],
+    "Int": [
+        {
+            "args": [("int",), ("real",), ("complex",)],
+            "return": "int32",
+            "kind_arg": True
+        }
+    ],
+    "StringLenTrim": [
+        {
+            "args": [("char",)],
+            "return": "int32",
+            "kind_arg": True
+        }
+    ],
+    "StringTrim": [
+        {
+            "args": [("char",)],
+            "ret_type_arg_idx": 0
+        }
+    ],
 }
 
 skip_create_func = ["Partition"]
 compile_time_only_fn = [
     "Epsilon",
     "Radix",
+    "IsContiguous",
+    "StorageSize",
     "Range",
     "Precision",
     "Rank",
@@ -777,6 +894,10 @@ compile_time_only_fn = [
     "BitSize",
     "NewLine",
     "Kind",
+    "MaxExponent",
+    "MinExponent",
+    "SameTypeAs",
+    "Digits",
 ]
 
 type_to_asr_type_check = {
@@ -809,6 +930,11 @@ def compute_arg_types(indent, no_of_args, args_arr):
     for i in range(no_of_args):
         src += indent + f"ASR::ttype_t *arg_type{i} = ASRUtils::expr_type({args_arr}[{i}]);\n"
 
+def compute_arg_kinds(indent, no_of_args):
+    global src
+    for i in range(no_of_args):
+        src += indent + f"int kind{i} = ASRUtils::extract_kind_from_ttype_t(arg_type{i});\n"
+
 def compute_arg_condition(no_of_args, args_lists):
     condition = []
     cond_in_msg = []
@@ -823,9 +949,16 @@ def compute_arg_condition(no_of_args, args_lists):
         cond_in_msg.append(", ".join(subcond_in_msg))
     return (f"({') || ('.join(condition)})", f"({') or ('.join(cond_in_msg)})")
 
+def compute_kind_condition(no_of_args):
+    condition = []
+    for i in range(1, no_of_args):
+        condition.append(f"kind0 == kind{i}")
+    return f"({') && ('.join(condition)})"
+
 def add_verify_arg_type_src(func_name):
     global src
     arg_infos = intrinsic_funcs_args[func_name]
+    same_kind_arg = arg_infos[0].get("same_kind_arg", False)
     no_of_args_msg = ""
     for i, arg_info in enumerate(arg_infos):
         args_lists = arg_info["args"]
@@ -838,6 +971,10 @@ def add_verify_arg_type_src(func_name):
         compute_arg_types(3 * indent, no_of_args, "x.m_args")
         condition, cond_in_msg = compute_arg_condition(no_of_args, args_lists)
         src += 3 * indent + f'ASRUtils::require_impl({condition}, "Unexpected args, {func_name} expects {cond_in_msg} as arguments", x.base.base.loc, diagnostics);\n'
+        if same_kind_arg:
+            compute_arg_kinds(3 * indent, same_kind_arg)
+            condition = compute_kind_condition(same_kind_arg)
+            src += 3 * indent + f'ASRUtils::require_impl({condition}, "Kind of all the arguments of {func_name} must be the same", x.base.base.loc, diagnostics);\n'
         src += 2 * indent + "}\n"
     src += 2 * indent + "else {\n"
     src += 3 * indent + f'ASRUtils::require_impl(false, "Unexpected number of args, {func_name} takes {no_of_args_msg} arguments, found " + std::to_string(x.n_args), x.base.base.loc, diagnostics);\n'
@@ -867,6 +1004,7 @@ def add_create_func_arg_type_src(func_name):
     for i, arg_info in enumerate(arg_infos):
         args_lists = arg_info["args"]
         kind_arg = arg_info.get("kind_arg", False)
+        same_kind_arg = arg_info.get("same_kind_arg", False)
         no_of_args = len(args_lists[0])
         no_of_args_msg += " or " if i > 0 else ""
         no_of_args_msg += f"{no_of_args + int(kind_arg)}"
@@ -878,6 +1016,13 @@ def add_create_func_arg_type_src(func_name):
         src += 4 * indent + f'append_error(diag, "Unexpected args, {func_name} expects {cond_in_msg} as arguments", loc);\n'
         src += 4 * indent + f'return nullptr;\n'
         src += 3 * indent + '}\n'
+        if same_kind_arg:
+            compute_arg_kinds(3 * indent, same_kind_arg)
+            condition = compute_kind_condition(same_kind_arg)
+            src += 3 * indent + f'if(!({condition}))' + ' {\n'
+            src += 4 * indent + f'append_error(diag, "Kind of all the arguments of {func_name} must be the same", loc);\n'
+            src += 4 * indent + f'return nullptr;\n'
+            src += 3 * indent + '}\n'
         src += 2 * indent + "}\n"
     src += 2 * indent + "else {\n"
     src += 3 * indent + f'append_error(diag, "Unexpected number of args, {func_name} takes {no_of_args_msg} arguments, found " + std::to_string(args.size()), loc);\n'
@@ -897,7 +1042,20 @@ def add_create_func_return_src(func_name):
     else:
         src += indent * 2 + "ASRUtils::ExprStmtDuplicator expr_duplicator(al);\n"
         src += indent * 2 + "expr_duplicator.allow_procedure_calls = true;\n"
-        src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
+        if ( ret_type_arg_idx == "dynamic"):
+            src += indent * 2 + f"int upper_kind = 0;\n"
+            src += indent * 2 + f"for(size_t i=0;i<args.size();i++){{\n"
+            src += indent * 3 + f"upper_kind = std::max(upper_kind,ASRUtils::extract_kind_from_ttype_t(expr_type(args[i])));\n"
+            src += indent * 2 + f"}}\n"
+            src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[0])));\n"
+            src += indent * 2 + f"set_kind_to_ttype_t(type_,upper_kind);\n"
+        else:
+            src += indent * 2 + "ASR::ttype_t* type_ = nullptr;\n"
+            src += indent * 2 + "if (use_experimental_simplifier) {\n"
+            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
+            src += indent * 2 + "} else {\n"
+            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
+            src += indent * 2 + "}\n"
         ret_type = "type_"
     kind_arg = arg_infos[0].get("kind_arg", False)
     src += indent * 2 + f"ASR::ttype_t *return_type = {ret_type};\n"
@@ -910,6 +1068,15 @@ def add_create_func_return_src(func_name):
         src += indent * 3 +     "}\n"
         src += indent * 3 +     "set_kind_to_ttype_t(return_type, kind);\n"
         src += indent * 2 + "}\n"
+    real_32_except_complex = arg_infos[0].get("real_32_except_complex", False)
+    if real_32_except_complex:
+        src += indent * 2 + "else { \n"
+        src += indent * 3 + "ASR::ttype_t* arg_type = ASRUtils::expr_type(args[0]);\n"
+        src += indent * 3 + "if (is_complex(*arg_type)) { \n"
+        src += indent * 4 + "int kind = ASRUtils::extract_kind_from_ttype_t(arg_type); \n"
+        src += indent * 4 + "set_kind_to_ttype_t(return_type, kind); \n"
+        src += indent * 3 + "} \n"
+        src += indent * 2 + "} \n"
     src += indent * 2 + "ASR::expr_t *m_value = nullptr;\n"
     src += indent * 2 + f"Vec<ASR::expr_t*> m_args; m_args.reserve(al, {no_of_args});\n"
     for _i in range(no_of_args):
@@ -925,6 +1092,28 @@ def add_create_func_return_src(func_name):
             "ASRUtils::expr_type(m_args[0]), m_args[0], return_type, m_value);\n"
 
     else:
+        src += indent * 2 + "if (use_experimental_simplifier) {\n"
+        src += indent * 3 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
+        src += indent * 4 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
+        src += indent * 4 +         "if (ASRUtils::is_array(type)) {\n"
+        src += indent * 5 +             "ASR::dimension_t* m_dims = nullptr;\n"
+        src += indent * 5 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
+        src += indent * 5 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
+        src +=                              "return_type, m_dims, n_dims, ASR::abiType::Source, false, "
+        src +=                              "ASR::array_physical_typeType::DescriptorArray);\n"
+        src += indent * 5 +             "break;\n"
+        src += indent * 4 +         "}\n"
+        src += indent * 3 +     "}\n"
+        src += indent * 2 + "}\n"
+        if ret_type_val:
+            src += indent * 2 + "else {\n"
+            src += indent * 3 +     "ASR::ttype_t* type = ASRUtils::expr_type(args[0]);\n"
+            src += indent * 3 +     "if (ASR::is_a<ASR::Array_t>(*type)) {\n"
+            src += indent * 4 +         "ASR::Array_t* e = ASR::down_cast<ASR::Array_t>(type);\n"
+            src += indent * 4 +         f"return_type = TYPE(ASR::make_Array_t(al, type->base.loc,  return_type, e->m_dims, e->n_dims, ASR::array_physical_typeType::FixedSizeArray));\n"
+            src += indent * 3 +     "}\n"
+            src += indent * 2 + "}\n"
+
         src += indent * 2 + "if (all_args_evaluated(m_args)) {\n"
         src += indent * 3 +     f"Vec<ASR::expr_t*> args_values; args_values.reserve(al, {no_of_args});\n"
         for _i in range(no_of_args):
@@ -973,6 +1162,7 @@ def get_registry_funcs_src():
 HEAD = """#ifndef LIBASR_PASS_INTRINSIC_FUNC_REG_UTIL_H
 #define LIBASR_PASS_INTRINSIC_FUNC_REG_UTIL_H
 
+#include <libasr/asr_utils.h>
 #include <libasr/pass/intrinsic_functions.h>
 
 namespace LCompilers {
