@@ -7,7 +7,6 @@
 #include <libasr/pass/pass_utils.h>
 
 #include <vector>
-#include <utility>
 
 namespace LCompilers {
 
@@ -63,13 +62,18 @@ class ReplaceInitExpr: public ASR::BaseExprReplacer<ReplaceInitExpr> {
         if( casted_type != nullptr ) {
             casted_type = ASRUtils::type_get_past_array(casted_type);
         }
-        PassUtils::ReplacerUtils::replace_ArrayConstructor(x, this,
-            remove_original_statement, result_vec,
-            perform_cast, cast_kind, casted_type);
+        if (ASRUtils::use_experimental_simplifier) {
+            PassUtils::ReplacerUtils::replace_ArrayConstructor_(al, x, result_var, result_vec,
+                current_scope, perform_cast, cast_kind, casted_type);
+        } else {
+            PassUtils::ReplacerUtils::replace_ArrayConstructor(x, this,
+                remove_original_statement, result_vec,
+                perform_cast, cast_kind, casted_type);
+        }
         *current_expr = nullptr;
     }
 
-    void replace_StructTypeConstructor(ASR::StructTypeConstructor_t* x) {
+    void replace_StructConstructor(ASR::StructConstructor_t* x) {
         if( symtab2decls.find(current_scope) == symtab2decls.end() ) {
             Vec<ASR::stmt_t*> result_vec_;
             result_vec_.reserve(al, 0);
@@ -77,7 +81,7 @@ class ReplaceInitExpr: public ASR::BaseExprReplacer<ReplaceInitExpr> {
         }
         Vec<ASR::stmt_t*>* result_vec = &symtab2decls[current_scope];
         bool remove_original_statement = false;
-        PassUtils::ReplacerUtils::replace_StructTypeConstructor(
+        PassUtils::ReplacerUtils::replace_StructConstructor(
             x, this, true, remove_original_statement, result_vec,
             perform_cast, cast_kind, casted_type);
         *current_expr = nullptr;
@@ -182,11 +186,13 @@ class InitExprVisitor : public ASR::CallReplacerOnExpressionsVisitor<InitExprVis
             }
             if( !(symbolic_value &&
                   (ASR::is_a<ASR::ArrayConstant_t>(*symbolic_value) ||
-                   ASR::is_a<ASR::StructTypeConstructor_t>(*symbolic_value) ||
+                   ASR::is_a<ASR::StructConstructor_t>(*symbolic_value) ||
                    ASR::is_a<ASR::ArrayConstructor_t>(*symbolic_value))) ||
                  (ASR::is_a<ASR::Module_t>(*asr_owner) &&
                   (ASR::is_a<ASR::ArrayConstant_t>(*symbolic_value) ||
-                  ASR::is_a<ASR::ArrayConstructor_t>(*symbolic_value)))) {
+                  ASR::is_a<ASR::ArrayConstructor_t>(*symbolic_value))) ||
+                (x.m_storage == ASR::storage_typeType::Save && 
+                ASR::is_a<ASR::Function_t>(*ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner)))) {
                 return ;
             }
 
