@@ -9,6 +9,7 @@
 #include <vector>
 #include <utility>
 #include <deque>
+#include <set>
 
 /*
 This ASR to ASR pass can be called whenever you want to avoid
@@ -312,11 +313,72 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
     EditProcedureReplacer(PassArrayByDataProcedureVisitor& v_):
      v(v_), current_scope(nullptr) {}
 
+    /*
+        We need this for the cases where pass array by data is possible for parent function and also
+        for the child function. Let's take the following example:
+        ```fortran
+        subroutine cobyla(x)
+            implicit none
+
+            real, intent(inout) :: x(:)
+            call evaluate(calcfc_internal)
+            contains
+
+            subroutine calcfc_internal(x_internal)
+                implicit none
+                real, intent(in) :: x_internal(:)
+            end subroutine calcfc_internal
+
+            subroutine evaluate(calcfc)
+                use, non_intrinsic :: pintrf_mod, only : OBJCON
+                implicit none
+                procedure(OBJCON) :: calcfc
+            end subroutine evaluate
+
+        end subroutine
+        ```
+
+        Here, cobyla is parent and calcfc_internal is child. What happens is by `bfs` we first
+        create `cobyla_real____0` and add it as a symbol in global scope. Then we visit `cobyla`,
+        create `calcfc_internal_real____0` as replacement of `calcfc_internal` and add it as a symbol
+        in scope of `cobyla`. Now, we visit `cobyla_real____0` and replace `calcfc_internal` with
+        `calcfc_internal_real____0`
+        
+        This means both symbols `cobyla` and `cobyla_real____0` have `calcfc_internal_real____0` as a symbol
+        but eventually `cobyla` is removed from scope.
+
+        In `proc2newproc` we map `cobyla(1) -> cobyla_real____0(2)` and `calcfc_internal(1.1) -> calcfc_internal_real____0(1.2)`
+        and also have `calcfc_internal(2.1) -> calcfc_internal_real____0(2.2)`. Where `(x)` represents symtab counter.
+
+        When it comes to replace `call evaluate(calcfc_internal(1.1))`, it gets replaced with `call evaluate(calcfc_internal_real____0(1.2))`
+        which eventually gets deleted. We need to actually replace it with `call evaluate(calcfc_internal_real____0(2.2))`, so what we do is
+        we resolve the symbol to the latest symbol in the scope.
+
+        From `calcfc_internal(1.1)`, we get `calcfc_internal_real____0(1.2)` and then we get the parent of `calcfc_internal_real____0(1.2)`,
+        i.e. `cobyla(1)`, resolve it to `cobyla_real____0(2)` and then get the symbol `calcfc_internal_real____0(2.2)` from it.
+    */
+    ASR::symbol_t* resolve_new_proc(ASR::symbol_t* old_sym) {
+        ASR::symbol_t* ext_sym = ASRUtils::symbol_get_past_external(old_sym);
+        if( v.proc2newproc.find(ext_sym) != v.proc2newproc.end() ) {
+            ASR::symbol_t* new_sym = v.proc2newproc[ext_sym].first;
+            ASR::asr_t* new_sym_parent = ASRUtils::symbol_parent_symtab(new_sym)->asr_owner;
+            if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) ) {
+                ASR::symbol_t* resolved_parent_sym = resolve_new_proc(ASR::down_cast<ASR::symbol_t>(new_sym_parent));
+                if ( resolved_parent_sym != nullptr ) {
+                    ASR::symbol_t* sym_to_return = ASRUtils::symbol_symtab(resolved_parent_sym)->get_symbol(ASRUtils::symbol_name(new_sym));
+                    return sym_to_return ? sym_to_return : new_sym;
+                }
+            }
+            return new_sym;
+        }
+        return nullptr;
+    }
+
     void replace_Var(ASR::Var_t* x) {
         ASR::symbol_t* x_sym_ = x->m_v;
         bool is_external = ASR::is_a<ASR::ExternalSymbol_t>(*x_sym_);
         if ( v.proc2newproc.find(ASRUtils::symbol_get_past_external(x_sym_)) != v.proc2newproc.end() ) {
-            x->m_v = v.proc2newproc[ASRUtils::symbol_get_past_external(x_sym_)].first;
+            x->m_v = resolve_new_proc(x_sym_);
             if( is_external ) {
                 ASR::ExternalSymbol_t* x_sym_ext = ASR::down_cast<ASR::ExternalSymbol_t>(x_sym_);
                 std::string new_func_sym_name = current_scope->get_unique_name(ASRUtils::symbol_name(
@@ -326,12 +388,13 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
                         s2c(v.al, new_func_sym_name), x->m_v, x_sym_ext->m_module_name,
                         x_sym_ext->m_scope_names, x_sym_ext->n_scope_names, ASRUtils::symbol_name(x->m_v),
                         x_sym_ext->m_access));
+                v.proc2newproc[x_sym_] = {new_func_sym_, {}};
                 current_scope->add_symbol(new_func_sym_name, new_func_sym_);
                 x->m_v = new_func_sym_;
             }
             return ;
         }
-
+        
         edit_symbol_pointer(v)
     }
 
@@ -339,15 +402,21 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
         ASR::BaseExprReplacer<EditProcedureReplacer>::replace_ArrayPhysicalCast(x);
         // TODO: Allow for DescriptorArray to DescriptorArray physical cast for allocatables
         // later on
+        x->m_old = ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg));
         if( (x->m_old == x->m_new &&
              x->m_old != ASR::array_physical_typeType::DescriptorArray) ||
             (x->m_old == x->m_new && x->m_old == ASR::array_physical_typeType::DescriptorArray &&
             (ASR::is_a<ASR::Allocatable_t>(*ASRUtils::expr_type(x->m_arg)) ||
-             ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x->m_arg)))) ||
-             x->m_old != ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg)) ) {
+             ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x->m_arg))))) {
             *current_expr = x->m_arg;
         } else {
-            x->m_old = ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg));
+            ASR::Array_t* arr = ASR::down_cast<ASR::Array_t>(ASRUtils::type_get_past_pointer(
+                  ASRUtils::type_get_past_allocatable(x->m_type)));
+            if (ASRUtils::is_dimension_empty(arr->m_dims, arr->n_dims)) {
+              arr->m_dims = ASR::down_cast<ASR::Array_t>(
+                  ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_pointer(
+                      ASRUtils::expr_type(x->m_arg))))->m_dims;
+            }
         }
     }
 
@@ -392,6 +461,27 @@ class EditProcedureVisitor: public ASR::CallReplacerOnExpressionsVisitor<EditPro
         ASR::CallReplacerOnExpressionsVisitor<EditProcedureVisitor>::visit_SubroutineCall(x);
     }
 
+    void visit_Module(const ASR::Module_t& x) {
+        for (auto it: x.m_symtab->get_scope()) {
+            if ( ASR::is_a<ASR::Variable_t>(*it.second) &&
+                ASR::down_cast<ASR::Variable_t>(it.second)->m_type_declaration ) {
+                ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(it.second);
+                ASR::symbol_t* resolved_type_dec = nullptr;
+                if ( v.proc2newproc.find(var->m_type_declaration) != v.proc2newproc.end() ) {
+                    resolved_type_dec = v.proc2newproc[var->m_type_declaration].first;
+                } else if ( v.proc2newproc.find(ASRUtils::symbol_get_past_external(var->m_type_declaration)) != v.proc2newproc.end() ) {
+                    resolved_type_dec = v.proc2newproc[ASRUtils::symbol_get_past_external(var->m_type_declaration)].first;
+                }
+                if ( resolved_type_dec ) {
+                    ASR::Function_t* fn = ASR::down_cast<ASR::Function_t>(resolved_type_dec);
+                    var->m_type_declaration = resolved_type_dec;
+                    var->m_type = fn->m_function_signature;
+                }
+            }
+        }
+        ASR::CallReplacerOnExpressionsVisitor<EditProcedureVisitor>::visit_Module(x);
+    }
+
 };
 
 /*
@@ -424,6 +514,24 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             std::set<ASR::symbol_t*>& not_to_be_erased_):
         al(al_), v(v_), not_to_be_erased(not_to_be_erased_) {}
 
+        // this is exactly the same as the one in EditProcedureReplacer
+        ASR::symbol_t* resolve_new_proc(ASR::symbol_t* old_sym) {
+            ASR::symbol_t* ext_sym = ASRUtils::symbol_get_past_external(old_sym);
+            if( v.proc2newproc.find(ext_sym) != v.proc2newproc.end() ) {
+                ASR::symbol_t* new_sym = v.proc2newproc[ext_sym].first;
+                ASR::asr_t* new_sym_parent = ASRUtils::symbol_parent_symtab(new_sym)->asr_owner;
+                if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) ) {
+                    ASR::symbol_t* resolved_parent_sym = resolve_new_proc(ASR::down_cast<ASR::symbol_t>(new_sym_parent));
+                    if ( resolved_parent_sym != nullptr ) {
+                        ASR::symbol_t* sym_to_return = ASRUtils::symbol_symtab(resolved_parent_sym)->get_symbol(ASRUtils::symbol_name(new_sym));
+                        return sym_to_return ? sym_to_return : new_sym;
+                    }
+                }
+                return new_sym;
+            }
+            return nullptr;
+        }
+
         template <typename T>
         void update_args_for_pass_arr_by_data_funcs_passed_as_callback(const T& x) {
             bool args_updated = false;
@@ -437,7 +545,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                         ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(expr);
                         ASR::symbol_t* sym = var->m_v;
                         if ( v.proc2newproc.find(sym) != v.proc2newproc.end() ) {
-                            ASR::symbol_t* new_var_sym = v.proc2newproc[sym].first;
+                            ASR::symbol_t* new_var_sym = resolve_new_proc(sym);
                             ASR::expr_t* new_var = ASRUtils::EXPR(ASR::make_Var_t(al, var->base.base.loc, new_var_sym));
                             {
                                 // update exisiting arg
@@ -484,8 +592,20 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             Vec<ASR::call_arg_t> new_args;
             new_args.reserve(al, n_args);
             for( size_t i = 0; i < n_args; i++ ) {
-                if (orig_args[i].m_value == nullptr ||
-                    std::find(indices.begin(), indices.end(), i) == indices.end()) {
+                if (orig_args[i].m_value == nullptr) {
+                    new_args.push_back(al, orig_args[i]);
+                    continue;
+                } else if (std::find(indices.begin(), indices.end(), i) == indices.end()) {
+                    ASR::expr_t* expr = orig_args[i].m_value;    
+                    if (ASR::is_a<ASR::Var_t>(*expr)) {
+                        ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(expr);
+                        ASR::symbol_t* sym = var->m_v;
+                        if ( v.proc2newproc.find(sym) != v.proc2newproc.end() ) {
+                            ASR::symbol_t* new_var_sym = v.proc2newproc[sym].first;
+                            ASR::expr_t* new_var = ASRUtils::EXPR(ASR::make_Var_t(al, var->base.base.loc, new_var_sym));
+                            orig_args[i].m_value = new_var;
+                        }
+                    }
                     new_args.push_back(al, orig_args[i]);
                     continue;
                 }
@@ -494,7 +614,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 ASR::ttype_t* orig_arg_type = ASRUtils::expr_type(orig_arg_i);
                 if( ASRUtils::is_array(orig_arg_type) ) {
                     ASR::Array_t* array_t = ASR::down_cast<ASR::Array_t>(
-                        ASRUtils::type_get_past_allocatable(orig_arg_type));
+                        ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_pointer(orig_arg_type)));
                     if( array_t->m_physical_type != ASR::array_physical_typeType::PointerToDataArray ) {
                         ASR::expr_t* physical_cast = ASRUtils::EXPR(ASRUtils::make_ArrayPhysicalCast_t_util(
                             al, orig_arg_i->base.loc, orig_arg_i, array_t->m_physical_type,
@@ -538,10 +658,48 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
 
         template <typename T>
         void visit_Call(const T& x) {
+            T& xx = const_cast<T&>(x);
             ASR::symbol_t* subrout_sym = x.m_name;
             bool is_external = ASR::is_a<ASR::ExternalSymbol_t>(*subrout_sym);
             subrout_sym = ASRUtils::symbol_get_past_external(subrout_sym);
 
+            if(ASR::is_a<ASR::Variable_t>(*ASRUtils::symbol_get_past_external(x.m_name))){
+                // Case: procedure(cb) :: call_back (Here call_back is variable of type cb which is a function)
+                ASR::Variable_t* variable = ASR::down_cast<ASR::Variable_t>(ASRUtils::symbol_get_past_external(x.m_name));
+                ASR::symbol_t* type_dec = variable->m_type_declaration;
+                subrout_sym = ASRUtils::symbol_get_past_external(type_dec);
+
+                // check if subrout_sym is present as value in proc2newproc
+                bool is_present = false;
+                std::vector<size_t> present_indices;
+                for ( auto it: v.proc2newproc ) {
+                    if (it.second.first == subrout_sym) {
+                        is_present = true;
+                        present_indices = it.second.second;
+                        break;
+                    }
+                }
+
+                if ( v.proc2newproc.find(subrout_sym) != v.proc2newproc.end() ) {
+                    ASR::symbol_t* new_x_name = resolve_new_proc(x.m_name);
+                    if ( new_x_name != nullptr ) {
+                        ASR::Function_t* new_func = ASR::down_cast<ASR::Function_t>(resolve_new_proc(subrout_sym));
+                        ASR::down_cast<ASR::Variable_t>(x.m_name)->m_type = new_func->m_function_signature;
+                        xx.m_name = new_x_name;
+                        xx.m_original_name = new_x_name;
+                        std::vector<size_t>& indices = v.proc2newproc[subrout_sym].second;
+                        Vec<ASR::call_arg_t> new_args = construct_new_args(x.n_args, x.m_args, indices);
+                        xx.m_args = new_args.p;
+                        xx.n_args = new_args.size();
+                        return;
+                    }
+                } else if ( is_present ) {
+                    Vec<ASR::call_arg_t> new_args = construct_new_args(x.n_args, x.m_args, present_indices);
+                    xx.m_args = new_args.p;
+                    xx.n_args = new_args.size();
+                    return;
+                }
+            }
             if( !can_edit_call(x.m_args, x.n_args) ) {
                 not_to_be_erased.insert(subrout_sym);
                 return ;
@@ -552,7 +710,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 return;
             }
 
-            ASR::symbol_t* new_func_sym = v.proc2newproc[subrout_sym].first;
+            ASR::symbol_t* new_func_sym = resolve_new_proc(subrout_sym);
             std::vector<size_t>& indices = v.proc2newproc[subrout_sym].second;
 
             Vec<ASR::call_arg_t> new_args = construct_new_args(x.n_args, x.m_args, indices);
@@ -586,19 +744,40 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 // clashes with user defined functions
                 char* new_func_sym_name = ASRUtils::symbol_name(new_func_sym);
                 if( current_scope->resolve_symbol(new_func_sym_name) == nullptr ) {
-                    new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
-                        ASR::make_ExternalSymbol_t(al, x.m_name->base.loc, func_ext_sym->m_parent_symtab,
-                            new_func_sym_name, new_func_sym, func_ext_sym->m_module_name,
-                            func_ext_sym->m_scope_names, func_ext_sym->n_scope_names, new_func_sym_name,
-                            func_ext_sym->m_access));
+                    if ( ASR::is_a<ASR::Variable_t>(*ASRUtils::symbol_get_past_external(func_ext_sym->m_external)) &&
+                         ASR::down_cast<ASR::Variable_t>(ASRUtils::symbol_get_past_external(func_ext_sym->m_external))->m_type_declaration ) {
+                        ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(ASRUtils::symbol_get_past_external(func_ext_sym->m_external));
+                        ASR::symbol_t* type_dec_sym = current_scope->resolve_symbol(ASRUtils::symbol_name(var->m_type_declaration));
+                        std::string module_name = func_ext_sym->m_module_name;
+
+                        if ( type_dec_sym && ASR::is_a<ASR::ExternalSymbol_t>(*type_dec_sym) ) {
+                            module_name = ASR::down_cast<ASR::ExternalSymbol_t>(type_dec_sym)->m_module_name;
+                        }
+                        new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
+                            ASR::make_ExternalSymbol_t(al, x.m_name->base.loc, func_ext_sym->m_parent_symtab,
+                                new_func_sym_name, new_func_sym, s2c(al, module_name),
+                                func_ext_sym->m_scope_names, func_ext_sym->n_scope_names, new_func_sym_name,
+                                func_ext_sym->m_access));
+                    } else {
+                        new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
+                            ASR::make_ExternalSymbol_t(al, x.m_name->base.loc, func_ext_sym->m_parent_symtab,
+                                new_func_sym_name, new_func_sym, func_ext_sym->m_module_name,
+                                func_ext_sym->m_scope_names, func_ext_sym->n_scope_names, new_func_sym_name,
+                                func_ext_sym->m_access));
+                    }
+
                     func_ext_sym->m_parent_symtab->add_symbol(new_func_sym_name, new_func_sym_);
                 } else {
                     new_func_sym_ = current_scope->resolve_symbol(new_func_sym_name);
                 }
             }
-            T& xx = const_cast<T&>(x);
-            xx.m_name = new_func_sym_;
-            xx.m_original_name = new_func_sym_;
+            if(!ASR::is_a<ASR::Variable_t>(*x.m_name)){
+                xx.m_name = new_func_sym_;
+                xx.m_original_name = new_func_sym_;
+            } else if(v.proc2newproc.find(x.m_name) != v.proc2newproc.end()){
+                xx.m_name = resolve_new_proc(x.m_name);
+                xx.m_original_name = resolve_new_proc(x.m_name);
+            }
             xx.m_args = new_args.p;
             xx.n_args = new_args.size();
         }
@@ -612,6 +791,53 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             visit_Call(x);
             ASR::ASRPassBaseWalkVisitor<EditProcedureCallsVisitor>::visit_FunctionCall(x);
         }
+
+        void visit_Var(const ASR::Var_t &x) {
+            if (v.proc2newproc.find(x.m_v) != v.proc2newproc.end()){
+                ASR::symbol_t* new_sym = v.proc2newproc[x.m_v].first;
+                ASR::Var_t& vv = const_cast<ASR::Var_t&>(x);
+                vv.m_v = new_sym;
+                return;
+            }
+            if(ASR::is_a<ASR::Variable_t>(*x.m_v)){
+                // Case: procedure(cb) :: call_back (Here call_back is variable of type cb which is a function)
+                ASR::Variable_t* variable = ASR::down_cast<ASR::Variable_t>(x.m_v);
+                ASR::symbol_t* type_dec = variable->m_type_declaration;
+                if (v.proc2newproc.find(ASRUtils::symbol_get_past_external(type_dec)) != v.proc2newproc.end() && 
+                        v.proc2newproc.find(type_dec) == v.proc2newproc.end()){
+                    ASR::symbol_t* new_func = resolve_new_proc(ASRUtils::symbol_get_past_external(type_dec));
+                    ASR::ExternalSymbol_t* x_sym_ext = ASR::down_cast<ASR::ExternalSymbol_t>(type_dec);
+                    std::string new_func_sym_name = x_sym_ext->m_parent_symtab->get_unique_name(ASRUtils::symbol_name(
+                        ASRUtils::symbol_get_past_external(type_dec)));
+                    ASR::symbol_t* new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
+                        ASR::make_ExternalSymbol_t(v.al, type_dec->base.loc, x_sym_ext->m_parent_symtab,
+                            s2c(v.al, new_func_sym_name), new_func, x_sym_ext->m_module_name,
+                            x_sym_ext->m_scope_names, x_sym_ext->n_scope_names, ASRUtils::symbol_name(new_func),
+                            x_sym_ext->m_access));
+                    v.proc2newproc[type_dec] = {new_func_sym_, {}};
+                    x_sym_ext->m_parent_symtab->add_symbol(new_func_sym_name, new_func_sym_);
+                }
+                if(type_dec && v.proc2newproc.find(type_dec) != v.proc2newproc.end() && 
+                    ASR::is_a<ASR::Function_t>(*ASRUtils::symbol_get_past_external(type_dec))){
+                    ASR::symbol_t* new_sym = resolve_new_proc(type_dec);
+                    if ( new_sym == nullptr ) {
+                        return;
+                    }
+                    ASR::Function_t * subrout = ASR::down_cast<ASR::Function_t>(ASRUtils::symbol_get_past_external(new_sym));
+                    std::string new_sym_name = current_scope->get_unique_name(ASRUtils::symbol_name(x.m_v));
+                    ASR::symbol_t* new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
+                        ASRUtils::make_Variable_t_util(v.al, x.m_v->base.loc, current_scope, s2c(v.al, new_sym_name), 
+                            variable->m_dependencies, variable->n_dependencies, variable->m_intent, 
+                            variable->m_symbolic_value, variable->m_value, variable->m_storage, subrout->m_function_signature, 
+                            new_sym, variable->m_abi, variable->m_access, variable->m_presence, variable->m_value_attr));
+                    v.proc2newproc[x.m_v] = {new_func_sym_, {}};
+                    current_scope->add_symbol(new_sym_name, new_func_sym_); 
+                    ASR::Var_t& vv = const_cast<ASR::Var_t&>(x);
+                    vv.m_v = new_func_sym_;
+                    not_to_be_erased.insert(new_func_sym_);
+                }
+            }
+        }
 };
 
 /*
@@ -650,7 +876,10 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
             for( auto& item: current_scope->get_scope() ) {
                 if (ASR::is_a<ASR::Function_t>(*item.second)) {
                     ASR::FunctionType_t* func_type = ASRUtils::get_FunctionType(item.second);
-                    if (func_type->n_arg_types >= 1 && ASR::is_a<ASR::Class_t>(*func_type->m_arg_types[0])) {
+                    SymbolTable* current_scope_copy = current_scope;
+                    visit_symbol(*item.second);
+                    current_scope = current_scope_copy;
+                    if (func_type->n_arg_types >= 1 && ASR::is_a<ASR::ClassType_t>(*func_type->m_arg_types[0])) {
                         continue;
                     }
                 }
@@ -666,7 +895,7 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
                 if ( ASR::is_a<ASR::Module_t>(*item.second) ||
                     ASR::is_a<ASR::Program_t>(*item.second) ||
                     ASR::is_a<ASR::Function_t>(*item.second) ||
-                    ASR::is_a<ASR::StructType_t>(*item.second) ||
+                    ASR::is_a<ASR::Struct_t>(*item.second) ||
                     ASR::is_a<ASR::Block_t>(*item.second)) {
                     SymbolTable* current_scope_copy = current_scope;
                     visit_symbol(*item.second);
@@ -699,7 +928,7 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
             visit_Unit(x);
         }
 
-        void visit_StructType(const ASR::StructType_t& x) {
+        void visit_Struct(const ASR::Struct_t& x) {
             visit_Unit(x);
         }
 
