@@ -58,16 +58,11 @@ public:
         Vec<ASR::expr_t*> idx_vars;
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
-        ASR::stmt_t* empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, loc,
-            nullptr, 0, nullptr, nullptr));
-        ASR::ttype_t *str_type_len_1 = ASRUtils::TYPE(ASR::make_Character_t(
-            al, loc, 1, 1, nullptr));
-        ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_Character_t(
-            al, loc, 1, 0, nullptr));
-        ASR::expr_t *space = ASRUtils::EXPR(ASR::make_StringConstant_t(
-            al, loc, s2c(al, " "), str_type_len_1));
+        ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
+            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len_2));
+        ASR::stmt_t* empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, loc, empty_space));
         for( int i = n_dims - 1; i >= 0; i-- ) {
             ASR::do_loop_head_t head;
             head.m_v = idx_vars[i];
@@ -90,14 +85,14 @@ public:
                     Vec<ASR::expr_t*> format_args;
                     format_args.reserve(al, 1);
                     format_args.push_back(al, string_format);
-                    print_stmt = ASRUtils::STMT(ASR::make_Print_t(al, loc,
-                        format_args.p, format_args.size(), nullptr, empty_space));
-                } else if (ASR::is_a<ASR::Character_t>(*ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_array(ASRUtils::expr_type(print_args[0]))))) {
-                    print_stmt = ASRUtils::STMT(ASR::make_Print_t(al, loc,
-                        print_args.p, print_args.size(), nullptr, empty_space));
+                    print_stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
+                        format_args.p, format_args.size()));
+                } else if (ASR::is_a<ASR::String_t>(*ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_array(ASRUtils::expr_type(print_args[0]))))) {
+                    print_stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
+                        print_args.p, print_args.size()));
                 } else {
-                    print_stmt = ASRUtils::STMT(ASR::make_Print_t(al, loc,
-                        print_args.p, print_args.size(), nullptr, space));
+                    print_stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
+                        print_args.p, print_args.size()));
                 }
                 doloop_body.push_back(al, print_stmt);
             } else {
@@ -117,9 +112,10 @@ public:
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::ttype_t *int32_type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4));
         ASR::expr_t* one = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, int32_type));
+        ASR::expr_t* lbound = PassUtils::get_bound(arr_expr, 1, "lbound", al);
         for (int n = 0; n < n_dims; n++) {
             ASR::expr_t* idx_var = idx_vars[n];
-            idx_var = one;
+            idx_var = lbound;
             for (int m = 0; m < m_dim_length; m++) {
                 ASR::expr_t* ref = PassUtils::create_array_ref(arr_expr, idx_var, al, current_scope);
                 print_body.push_back(ref);
@@ -144,8 +140,8 @@ public:
         print_args.push_back(al, string_format);
         ASR::stmt_t* statement = nullptr;
         if (_type == ASR::stmtType::Print) {
-            statement = ASRUtils::STMT(ASR::make_Print_t(al, loc,
-                print_args.p, print_args.size(), nullptr, nullptr));
+            statement = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
+                print_args.p, print_args.size()));
         } else if (_type == ASR::stmtType::FileWrite) {
             statement = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc, 0, unit,
                 nullptr, nullptr, nullptr, print_args.p, print_args.size(), separator, end, overloaded));
@@ -155,13 +151,17 @@ public:
     }
 
     void visit_Print(const ASR::Print_t& x) {
-        std::vector<ASR::expr_t*> print_body;
-        ASR::stmt_t* empty_print_endl;
-        ASR::stmt_t* print_stmt;
-        if (x.n_values > 0 && ASR::is_a<ASR::StringFormat_t>(*x.m_values[0])) {
-            empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-                nullptr, 0, nullptr, nullptr));
-            ASR::StringFormat_t* format = ASR::down_cast<ASR::StringFormat_t>(x.m_values[0]);
+        LCOMPILERS_ASSERT(ASR::is_a<ASR::String_t>(*ASRUtils::expr_type(x.m_text)));
+        if (ASR::is_a<ASR::StringFormat_t>(*x.m_text)) {
+            std::vector<ASR::expr_t*> print_body;
+            ASR::stmt_t* empty_print_endl;
+            ASR::stmt_t* print_stmt;
+            ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
+            al, x.base.base.loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+            ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
+            al, x.base.base.loc, s2c(al, ""), str_type_len_2));
+            empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc, empty_space));
+            ASR::StringFormat_t* format = ASR::down_cast<ASR::StringFormat_t>(x.m_text);
             for (size_t i=0; i<format->n_args; i++) {
                 if (PassUtils::is_array(format->m_args[i])) {
                     if (ASRUtils::is_fixed_size_array(ASRUtils::expr_type(format->m_args[i]))) {
@@ -184,74 +184,8 @@ public:
                 pass_result.push_back(al, print_stmt);
             }
             return;
-        }
-        ASR::ttype_t *str_type_len_1 = ASRUtils::TYPE(ASR::make_Character_t(
-            al, x.base.base.loc, 1, 1, nullptr));
-        ASR::expr_t *space = ASRUtils::EXPR(ASR::make_StringConstant_t(
-        al, x.base.base.loc, s2c(al, " "), str_type_len_1));
-        ASR::expr_t *backspace = ASRUtils::EXPR(ASR::make_StringConstant_t(
-        al, x.base.base.loc, s2c(al, "\b"), str_type_len_1));
-        ASR::stmt_t* back = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-            nullptr, 0, nullptr, backspace));
-        for (size_t i=0; i<x.n_values; i++) {
-            // DIVERGENCE between LFortran and LPython
-            // If a pointer array variable is provided
-            // then it will be printed as a normal array.
-            if (PassUtils::is_array(x.m_values[i])) {
-                if (print_body.size() > 0) {
-                    Vec<ASR::expr_t*> body;
-                    body.reserve(al, print_body.size());
-                    for (size_t j=0; j<print_body.size(); j++) {
-                        body.push_back(al, print_body[j]);
-                    }
-                    if (x.m_separator) {
-                        print_stmt = ASRUtils::STMT(ASR::make_Print_t(
-                            al, x.base.base.loc, body.p, body.size(),
-                            x.m_separator, x.m_separator));
-                    } else {
-                        print_stmt = ASRUtils::STMT(ASR::make_Print_t(
-                            al, x.base.base.loc, body.p, body.size(),
-                            nullptr, space));
-                    }
-                    pass_result.push_back(al, print_stmt);
-                    print_body.clear();
-                }
-                print_stmt = print_array_using_doloop(x.m_values[i], nullptr, x.base.base.loc);
-                pass_result.push_back(al, print_stmt);
-                pass_result.push_back(al, back);
-                if (x.m_separator) {
-                    if (i == x.n_values - 1) {
-                        empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-                            nullptr, 0, nullptr, x.m_end));
-                    } else {
-                        empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-                            nullptr, 0, nullptr, x.m_separator));
-                    }
-                } else {
-                    if (i == x.n_values - 1) {
-                        empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-                            nullptr, 0, nullptr, x.m_end));
-                    } else {
-                        empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc,
-                            nullptr, 0, nullptr, nullptr));
-                    }
-                }
-                pass_result.push_back(al, empty_print_endl);
-            } else {
-                print_body.push_back(x.m_values[i]);
-            }
-        }
-        if (print_body.size() > 0) {
-            Vec<ASR::expr_t*> body;
-            body.reserve(al, print_body.size());
-            for (size_t j=0; j<print_body.size(); j++) {
-                body.push_back(al, print_body[j]);
-            }
-            print_stmt = ASRUtils::STMT(ASR::make_Print_t(
-                al, x.base.base.loc, body.p, body.size(),
-                x.m_separator, x.m_end));
-            pass_result.push_back(al, print_stmt);
-            print_body.clear();
+        } else {
+            remove_original_stmt = false;
         }
     }
 
@@ -260,8 +194,8 @@ public:
         Vec<ASR::expr_t*> idx_vars;
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
-        ASR::ttype_t *str_type_len = ASRUtils::TYPE(ASR::make_Character_t(
-            al, loc, 1, 0, nullptr));
+        ASR::ttype_t *str_type_len = ASRUtils::TYPE(ASR::make_String_t(
+            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len));
         ASR::stmt_t* empty_file_write_endl = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc,
@@ -316,64 +250,67 @@ public:
         write_body.clear();
     }
 
-    void visit_FileWrite(const ASR::FileWrite_t& x) {
-        if (x.m_unit && ASRUtils::is_character(*ASRUtils::expr_type(x.m_unit))) {
-            // Skip for character write
-            return;
-        }
-        std::vector<ASR::expr_t*> write_body;
-        ASR::stmt_t* write_stmt;
-        ASR::stmt_t* empty_file_write_endl = ASRUtils::STMT(ASR::make_FileWrite_t(al, x.base.base.loc,
-            x.m_label, x.m_unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr));
-        if(x.m_values && x.m_values[0] != nullptr && ASR::is_a<ASR::StringFormat_t>(*x.m_values[0])){
-            ASR::StringFormat_t* format = ASR::down_cast<ASR::StringFormat_t>(x.m_values[0]);
-            for (size_t i=0; i<format->n_args; i++) {
-                if (PassUtils::is_array(format->m_args[i])) {
-                    if (ASRUtils::is_fixed_size_array(ASRUtils::expr_type(format->m_args[i]))) {
-                        print_fixed_sized_array(format->m_args[i], write_body, x.base.base.loc);
-                    } else {
-                        if (write_body.size() > 0) {
-                            write_stmt = create_formatstmt(write_body, format,
-                                x.base.base.loc, ASR::stmtType::FileWrite, x.m_unit, x.m_separator,
-                                x.m_end, x.m_overloaded);
-                            pass_result.push_back(al, write_stmt);
-                        }
-                        write_stmt = write_array_using_doloop(format->m_args[i], format, x.m_unit, x.base.base.loc);
-                        pass_result.push_back(al, write_stmt);
-                        pass_result.push_back(al, empty_file_write_endl);
-                    }
-                } else {
-                    write_body.push_back(format->m_args[i]);
-                }
-            }
-            if (write_body.size() > 0) {
-                write_stmt = create_formatstmt(write_body, format, x.base.base.loc,
-                    ASR::stmtType::FileWrite, x.m_unit, x.m_separator,
-                    x.m_end, x.m_overloaded);
-                pass_result.push_back(al, write_stmt);
-            }
-            return;
-        }
-        for (size_t i=0; i<x.n_values; i++) {
-            // DIVERGENCE between LFortran and LPython
-            // If a pointer array variable is provided
-            // then it will be printed as a normal array.
-            if (PassUtils::is_array(x.m_values[i])) {
-                if (write_body.size() > 0) {
-                    print_args_apart_from_arrays(write_body, x);
-                    pass_result.push_back(al, empty_file_write_endl);
-                }
-                write_stmt = write_array_using_doloop(x.m_values[i], nullptr, x.m_unit, x.base.base.loc);
-                pass_result.push_back(al, write_stmt);
-                pass_result.push_back(al, empty_file_write_endl);
-            } else {
-                write_body.push_back(x.m_values[i]);
-            }
-        }
-        if (write_body.size() > 0) {
-            print_args_apart_from_arrays(write_body, x);
-        }
-    }
+    // TODO :: CREATE write visitor to loop on arrays of type `structType` only,
+    // otherwise arrays are handled by backend.
+
+    // void visit_FileWrite(const ASR::FileWrite_t& x) {
+    //     if (x.m_unit && ASRUtils::is_character(*ASRUtils::expr_type(x.m_unit))) {
+    //         // Skip for character write
+    //         return;
+    //     }
+    //     std::vector<ASR::expr_t*> write_body;
+    //     ASR::stmt_t* write_stmt;
+    //     ASR::stmt_t* empty_file_write_endl = ASRUtils::STMT(ASR::make_FileWrite_t(al, x.base.base.loc,
+    //         x.m_label, x.m_unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr));
+    //     if(x.m_values && x.m_values[0] != nullptr && ASR::is_a<ASR::StringFormat_t>(*x.m_values[0])){
+    //         ASR::StringFormat_t* format = ASR::down_cast<ASR::StringFormat_t>(x.m_values[0]);
+    //         for (size_t i=0; i<format->n_args; i++) {
+    //             if (PassUtils::is_array(format->m_args[i])) {
+    //                 if (ASRUtils::is_fixed_size_array(ASRUtils::expr_type(format->m_args[i]))) {
+    //                     print_fixed_sized_array(format->m_args[i], write_body, x.base.base.loc);
+    //                 } else {
+    //                     if (write_body.size() > 0) {
+    //                         write_stmt = create_formatstmt(write_body, format,
+    //                             x.base.base.loc, ASR::stmtType::FileWrite, x.m_unit, x.m_separator,
+    //                             x.m_end, x.m_overloaded);
+    //                         pass_result.push_back(al, write_stmt);
+    //                     }
+    //                     write_stmt = write_array_using_doloop(format->m_args[i], format, x.m_unit, x.base.base.loc);
+    //                     pass_result.push_back(al, write_stmt);
+    //                     pass_result.push_back(al, empty_file_write_endl);
+    //                 }
+    //             } else {
+    //                 write_body.push_back(format->m_args[i]);
+    //             }
+    //         }
+    //         if (write_body.size() > 0) {
+    //             write_stmt = create_formatstmt(write_body, format, x.base.base.loc,
+    //                 ASR::stmtType::FileWrite, x.m_unit, x.m_separator,
+    //                 x.m_end, x.m_overloaded);
+    //             pass_result.push_back(al, write_stmt);
+    //         }
+    //         return;
+    //     }
+    //     for (size_t i=0; i<x.n_values; i++) {
+    //         // DIVERGENCE between LFortran and LPython
+    //         // If a pointer array variable is provided
+    //         // then it will be printed as a normal array.
+    //         if (PassUtils::is_array(x.m_values[i])) {
+    //             if (write_body.size() > 0) {
+    //                 print_args_apart_from_arrays(write_body, x);
+    //                 pass_result.push_back(al, empty_file_write_endl);
+    //             }
+    //             write_stmt = write_array_using_doloop(x.m_values[i], nullptr, x.m_unit, x.base.base.loc);
+    //             pass_result.push_back(al, write_stmt);
+    //             pass_result.push_back(al, empty_file_write_endl);
+    //         } else {
+    //             write_body.push_back(x.m_values[i]);
+    //         }
+    //     }
+    //     if (write_body.size() > 0) {
+    //         print_args_apart_from_arrays(write_body, x);
+    //     }
+    // }
 
 };
 
