@@ -33,7 +33,7 @@ class CreateFunctionFromSubroutine: public PassUtils::PassVisitor<CreateFunction
                 if (is_a<ASR::Function_t>(*item.second)) {
                     PassUtils::handle_fn_return_var(al,
                         ASR::down_cast<ASR::Function_t>(item.second),
-                        PassUtils::is_aggregate_or_array_type);
+                        PassUtils::is_aggregate_or_array_or_nonPrimitive_type);
                 }
             }
 
@@ -58,7 +58,7 @@ class CreateFunctionFromSubroutine: public PassUtils::PassVisitor<CreateFunction
                 if (is_a<ASR::Function_t>(*item.second)) {
                     PassUtils::handle_fn_return_var(al,
                         ASR::down_cast<ASR::Function_t>(item.second),
-                        PassUtils::is_aggregate_or_array_type);
+                        PassUtils::is_aggregate_or_array_or_nonPrimitive_type);
                 }
             }
 
@@ -79,7 +79,7 @@ class CreateFunctionFromSubroutine: public PassUtils::PassVisitor<CreateFunction
                 if (is_a<ASR::Function_t>(*item.second)) {
                     PassUtils::handle_fn_return_var(al,
                         ASR::down_cast<ASR::Function_t>(item.second),
-                        PassUtils::is_aggregate_or_array_type);
+                        PassUtils::is_aggregate_or_array_or_nonPrimitive_type);
                 }
             }
 
@@ -169,13 +169,14 @@ class ReplaceFunctionCallWithSubroutineCall:
             }
 
             bool is_return_var_handled = false;
-            ASR::symbol_t *fn_name = ASRUtils::symbol_get_past_external(x->m_name);
+            ASR::symbol_t *fn_name = ASRUtils::symbol_get_past_ClassProcedure(
+                ASRUtils::symbol_get_past_external(x->m_name));
             if (ASR::is_a<ASR::Function_t>(*fn_name)) {
                 ASR::Function_t *fn = ASR::down_cast<ASR::Function_t>(fn_name);
                 is_return_var_handled = fn->m_return_var == nullptr;
             }
             if (is_return_var_handled) {
-                ASR::ttype_t* result_var_type = x->m_type;
+                ASR::ttype_t* result_var_type = ASRUtils::duplicate_type(al, x->m_type);
                 bool is_allocatable = false;
                 bool is_func_call_allocatable = false;
                 bool is_result_var_allocatable = false;
@@ -296,17 +297,39 @@ class ReplaceFunctionCallWithSubroutineCall:
                     current_expr = current_expr_copy_9;
                     s_args.push_back(al, x->m_args[i]);
                 }
+                if(is_func_call_allocatable){ //Make sure to deallocate the argument that will hold the return of function.
+                    Vec<ASR::expr_t*> to_be_deallocated;
+                    to_be_deallocated.reserve(al, 1);
+                    to_be_deallocated.push_back(al, *current_expr);
+                    pass_result.push_back(al, ASRUtils::STMT(
+                        ASR::make_ImplicitDeallocate_t(al, (*current_expr)->base.loc,
+                        to_be_deallocated.p, to_be_deallocated.size())));
+                }
                 ASR::call_arg_t result_arg;
                 result_arg.loc = loc;
                 result_arg.m_value = *current_expr;
                 s_args.push_back(al, result_arg);
                 ASR::stmt_t* subrout_call = ASRUtils::STMT(ASRUtils::make_SubroutineCall_t_util(al, loc,
-                                                x->m_name, nullptr, s_args.p, s_args.size(), nullptr,
+                                                x->m_name, nullptr, s_args.p, s_args.size(), x->m_dt,
                                                 nullptr, false, false));
                 pass_result.push_back(al, subrout_call);
             }
         }
 
+        void replace_ArrayPhysicalCast(ASR::ArrayPhysicalCast_t* x) {
+            ASR::BaseExprReplacer<ReplaceFunctionCallWithSubroutineCall>::replace_ArrayPhysicalCast(x);
+            if( (x->m_old == x->m_new &&
+                x->m_old != ASR::array_physical_typeType::DescriptorArray) ||
+                (x->m_old == x->m_new && x->m_old == ASR::array_physical_typeType::DescriptorArray &&
+                (ASR::is_a<ASR::Allocatable_t>(*ASRUtils::expr_type(x->m_arg)) ||
+                ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x->m_arg)))) ||
+                x->m_old != ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg)) ) {
+                *current_expr = x->m_arg;
+            } else {
+                x->m_old = ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg));
+            }
+        }
+
 };
 
 class ReplaceFunctionCallWithSubroutineCallVisitor:
@@ -365,7 +388,8 @@ class ReplaceFunctionCallWithSubroutineCallVisitor:
         void visit_Assignment(const ASR::Assignment_t &x) {
             if( (ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x.m_target)) &&
                 ASR::is_a<ASR::GetPointer_t>(*x.m_value)) ||
-                (ASR::is_a<ASR::ArrayConstant_t>(*x.m_value))) {
+                (ASR::is_a<ASR::ArrayConstant_t>(*x.m_value) ||
+                ASR::is_a<ASR::ArrayConstructor_t>(*x.m_value)) ) {
                 return ;
             }
 
