@@ -6,7 +6,6 @@
 #include <libasr/pass/unique_symbols.h>
 #include <libasr/pass/pass_utils.h>
 #include <unordered_map>
-#include <set>
 #include<unordered_set>
 
 
@@ -242,19 +241,19 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
         }
     }
 
-    void visit_StructType(const ASR::StructType_t &x) {
+    void visit_Struct(const ASR::Struct_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_EnumType(const ASR::EnumType_t &x) {
+    void visit_Enum(const ASR::Enum_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_UnionType(const ASR::UnionType_t &x) {
+    void visit_Union(const ASR::Union_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_ClassType(const ASR::ClassType_t &x) {
+    void visit_Class(const ASR::Class_t &x) {
         visit_symbols_2(x);
     }
 
@@ -444,15 +443,15 @@ class UniqueSymbolVisitor: public ASR::BaseWalkVisitor<UniqueSymbolVisitor> {
         current_scope = current_scope_copy;
     }
 
-    void visit_StructType(const ASR::StructType_t &x) {
+    void visit_Struct(const ASR::Struct_t &x) {
         update_symbols_2(x);
     }
 
-    void visit_EnumType(const ASR::EnumType_t &x) {
+    void visit_Enum(const ASR::Enum_t &x) {
         update_symbols_2(x);
     }
 
-    void visit_UnionType(const ASR::UnionType_t &x) {
+    void visit_Union(const ASR::Union_t &x) {
         update_symbols_2(x);
     }
 
@@ -472,8 +471,8 @@ class UniqueSymbolVisitor: public ASR::BaseWalkVisitor<UniqueSymbolVisitor> {
         }
     }
 
-    void visit_ClassType(const ASR::ClassType_t &x) {
-        ASR::ClassType_t& xx = const_cast<ASR::ClassType_t&>(x);
+    void visit_Class(const ASR::Class_t &x) {
+        ASR::Class_t& xx = const_cast<ASR::Class_t&>(x);
         ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
         if (sym_to_new_name.find(sym) != sym_to_new_name.end()) {
             xx.m_name = s2c(al, sym_to_new_name[sym]);
