@@ -85,8 +85,13 @@ public:
         this->replace_expr(x->m_right); \
         ASR::expr_t* right = *current_expr; \
         current_expr = current_expr_copy; \
+        ASR::ttype_t* return_type;\
+        /* Both sides are primitives, OR one side is primitive and the other is ArrayBroadCast, OR both arrays*/ \
+        bool return_is_primitve =!ASRUtils::is_array(ASRUtils::expr_type(x->m_left)) || \
+            !ASRUtils::is_array(ASRUtils::expr_type(x->m_right)); \
+        return_type = return_is_primitve? ASRUtils::type_get_past_array(ASRUtils::expr_type(x->m_left)) : x->m_type;\
         *current_expr = ASRUtils::EXPR(ASR::Constructor(al, x->base.base.loc, \
-            left, x->m_op, right, x->m_type, nullptr)); \
+            left, x->m_op, right, return_type, nullptr)); \
 
     void replace_IntegerBinOp(ASR::IntegerBinOp_t* x) {
         BinOpReplacement(make_IntegerBinOp_t)
@@ -254,7 +259,7 @@ public:
                                         arr_section->m_v,
                                         args.p,
                                         args.size(),
-                                        ASRUtils::type_get_past_array_pointer_allocatable(
+                                        ASRUtils::extract_type(
                                             ASRUtils::expr_type((arr_section_var))),
                                         ASR::arraystorageType::ColMajor,
                                         nullptr));
@@ -441,7 +446,7 @@ public:
             log_bin_op = ASR::down_cast<ASR::LogicalBinOp_t>(test);
             left = log_bin_op->m_left;
         } else {
-            throw LCompilersException("Unsupported type, " + std::to_string(test->type));
+            throw LCompilersException("Unsupported type, " + ASRUtils::type_to_str_python(ASRUtils::expr_type(test)));
         }
 
         // create index variables.
@@ -452,13 +457,20 @@ public:
         if (ASR::is_a<ASR::FunctionCall_t>(*left)) {
             // Create an assignment `return_var = left` and replace function call with return_var
             ASR::FunctionCall_t* fc = ASR::down_cast<ASR::FunctionCall_t>(left);
-            uint64_t h = get_hash((ASR::asr_t*) fc->m_name);
-            ASR::Function_t* fn = ASR::down_cast<ASR::Function_t>(fc->m_name);
+            ASR::symbol_t* fn_sym = ASRUtils::symbol_get_past_external(fc->m_name);
+            uint64_t h = get_hash((ASR::asr_t*) fn_sym);
+            ASR::Function_t* fn = ASR::down_cast<ASR::Function_t>(fn_sym);
+            ASR::FunctionType_t* fn_type = ASR::down_cast<ASR::FunctionType_t>(fn->m_function_signature);
             ASR::expr_t* return_var_expr = fn->m_return_var;
             ASR::Variable_t* return_var = ASRUtils::EXPR2VAR(return_var_expr);
+            ASR::ttype_t* var_type = return_var->m_type;
+            if (fn_type->m_elemental) {
+                // here var_type will be a scalar type, we need to convert it to an array type
+                var_type = fc->m_type;
+            }
             ASR::expr_t* new_return_var_expr = PassUtils::create_var(1,
                 return_var->m_name, return_var->base.base.loc,
-                return_var->m_type, al, current_scope);
+                var_type, al, current_scope);
             assign_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, loc, new_return_var_expr, left, nullptr));
             opt_left = new_return_var_expr;
             return_var_hash[h] = opt_left;
