@@ -68,7 +68,7 @@ typedef double _Complex double_complex_t;
 LFORTRAN_API double _lfortran_sum(int n, double *v);
 LFORTRAN_API void _lfortran_random_number(int n, double *v);
 LFORTRAN_API void _lfortran_init_random_clock();
-LFORTRAN_API void _lfortran_init_random_seed(unsigned seed);
+LFORTRAN_API int _lfortran_init_random_seed(unsigned seed);
 LFORTRAN_API double _lfortran_random();
 LFORTRAN_API int _lfortran_randrange(int lower, int upper);
 LFORTRAN_API int _lfortran_random_int(int lower, int upper);
@@ -105,8 +105,8 @@ LFORTRAN_API float_complex_t _lfortran_cexp(float_complex_t x);
 LFORTRAN_API double_complex_t _lfortran_zexp(double_complex_t x);
 LFORTRAN_API float _lfortran_slog(float x);
 LFORTRAN_API double _lfortran_dlog(double x);
-LFORTRAN_API bool _lfortran_rsp_is_nan(float x);
-LFORTRAN_API bool _lfortran_rdp_is_nan(double x);
+LFORTRAN_API bool _lfortran_sis_nan(float x);
+LFORTRAN_API bool _lfortran_dis_nan(double x);
 LFORTRAN_API float_complex_t _lfortran_clog(float_complex_t x);
 LFORTRAN_API double_complex_t _lfortran_zlog(double_complex_t x);
 LFORTRAN_API float _lfortran_serf(float x);
@@ -195,7 +195,8 @@ LFORTRAN_API int32_t _lpython_bit_length8(int64_t num);
 LFORTRAN_API void _lfortran_strrepeat(char** s, int32_t n, char** dest);
 LFORTRAN_API char* _lfortran_strrepeat_c(char* s, int32_t n);
 LFORTRAN_API void _lfortran_strcat(char** s1, char** s2, char** dest);
-LFORTRAN_API void _lfortran_strcpy(char** x, char *y, int8_t free_target);
+LFORTRAN_API void _lfortran_strcpy_pointer_string(char** x, char *y);
+LFORTRAN_API void _lfortran_strcpy_descriptor_string(char** x, char *y, int64_t* x_string_size, int64_t* x_string_capacity);
 LFORTRAN_API int32_t _lfortran_str_len(char** s);
 LFORTRAN_API int _lfortran_str_ord(char** s);
 LFORTRAN_API int _lfortran_str_ord_c(char* s);
@@ -206,9 +207,9 @@ LFORTRAN_API void _lfortran_memset(void* s, int32_t c, int32_t size);
 LFORTRAN_API int8_t* _lfortran_realloc(int8_t* ptr, int32_t size);
 LFORTRAN_API int8_t* _lfortran_calloc(int32_t count, int32_t size);
 LFORTRAN_API void _lfortran_free(char* ptr);
-LFORTRAN_API void _lfortran_alloc(char** ptr, int32_t len);
+LFORTRAN_API void _lfortran_alloc(char** ptr, int32_t len, int64_t* size, int64_t* capacity);
 LFORTRAN_API void _lfortran_string_init(int size_plus_one, char *s);
-LFORTRAN_API char* _lfortran_str_item(char* s, int32_t idx);
+LFORTRAN_API char* _lfortran_str_item(char* s, int64_t idx);
 LFORTRAN_API char* _lfortran_str_copy(char* s, int32_t idx1, int32_t idx2); // idx1 and idx2 both start from 1
 LFORTRAN_API char* _lfortran_str_slice(char* s, int32_t idx1, int32_t idx2, int32_t step,
                         bool idx1_present, bool idx2_present);
@@ -220,20 +221,24 @@ LFORTRAN_API int64_t _lfortran_mvbits64(int64_t from, int32_t frompos,
                                         int32_t len, int64_t to, int32_t topos);
 LFORTRAN_API int32_t _lfortran_ibits32(int32_t i, int32_t pos, int32_t len);
 LFORTRAN_API int64_t _lfortran_ibits64(int64_t i, int32_t pos, int32_t len);
-LFORTRAN_API void _lfortran_cpu_time(double *t);
+LFORTRAN_API double _lfortran_d_cpu_time();
+LFORTRAN_API float _lfortran_s_cpu_time();
 LFORTRAN_API void _lfortran_i32sys_clock(
         int32_t *count, int32_t *rate, int32_t *max);
 LFORTRAN_API void _lfortran_i64sys_clock(
         uint64_t *count, int64_t *rate, int64_t *max);
 LFORTRAN_API void _lfortran_i64r64sys_clock(
         uint64_t *count, double *rate, int64_t *max);
-LFORTRAN_API double _lfortran_time();
+LFORTRAN_API char* _lfortran_date();
+LFORTRAN_API char* _lfortran_time();
+LFORTRAN_API char* _lfortran_zone();
+LFORTRAN_API int32_t _lfortran_values();
 LFORTRAN_API float _lfortran_sp_rand_num();
 LFORTRAN_API double _lfortran_dp_rand_num();
 LFORTRAN_API int64_t _lpython_open(char *path, char *flags);
 LFORTRAN_API int64_t _lfortran_open(int32_t unit_num, char *f_name, char *status, char* form);
 LFORTRAN_API void _lfortran_flush(int32_t unit_num);
-LFORTRAN_API void _lfortran_inquire(char *f_name, bool *exists, int32_t unit_num, bool *opened);
+LFORTRAN_API void _lfortran_inquire(char *f_name, bool *exists, int32_t unit_num, bool *opened, int32_t *size);
 LFORTRAN_API void _lfortran_formatted_read(int32_t unit_num, int32_t* iostat, int32_t* chunk, char* fmt, int32_t no_of_args, ...);
 LFORTRAN_API char* _lpython_read(int64_t fd, int64_t n);
 LFORTRAN_API void _lfortran_read_int32(int32_t *p, int32_t unit_num);
@@ -244,21 +249,26 @@ LFORTRAN_API void _lfortran_read_float(float *p, int32_t unit_num);
 LFORTRAN_API void _lfortran_read_array_float(float *p, int array_size, int32_t unit_num);
 LFORTRAN_API void _lfortran_read_array_double(double *p, int array_size, int32_t unit_num);
 LFORTRAN_API void _lfortran_read_char(char **p, int32_t unit_num);
-LFORTRAN_API void _lfortran_string_write(char **str, int32_t* iostat, const char *format, ...);
+LFORTRAN_API void _lfortran_string_write(char **str, int64_t* size, int64_t* capacity, int32_t* iostat, const char *format, ...);
 LFORTRAN_API void _lfortran_file_write(int32_t unit_num, int32_t* iostat, const char *format, ...);
-LFORTRAN_API void _lfortran_string_read(char *str, char *format, int *i);
+LFORTRAN_API void _lfortran_string_read_i32(char *str, char *format, int32_t *i);
+LFORTRAN_API void _lfortran_string_read_i64(char *str, char *format, int64_t *i);
+LFORTRAN_API void _lfortran_string_read_f32(char *str, char *format, float *f);
+LFORTRAN_API void _lfortran_string_read_f64(char *str, char *format, double *f);
+LFORTRAN_API void _lfortran_string_read_str(char *str, char *format, char **s);
 LFORTRAN_API void _lfortran_empty_read(int32_t unit_num, int32_t* iostat);
 LFORTRAN_API void _lpython_close(int64_t fd);
-LFORTRAN_API void _lfortran_close(int32_t unit_num);
+LFORTRAN_API void _lfortran_close(int32_t unit_num, char* status);
 LFORTRAN_API int32_t _lfortran_ichar(char *c);
 LFORTRAN_API int32_t _lfortran_iachar(char *c);
-LFORTRAN_API int32_t _lfortran_all(bool *mask, int32_t n);
 LFORTRAN_API void _lpython_set_argv(int32_t argc_1, char *argv_1[]);
+LFORTRAN_API void _lpython_free_argv();
 LFORTRAN_API int32_t _lpython_get_argc();
 LFORTRAN_API char *_lpython_get_argv(int32_t index);
 LFORTRAN_API void _lpython_call_initial_functions(int32_t argc_1, char *argv_1[]);
 LFORTRAN_API void print_stacktrace_addresses(char *filename, bool use_colors);
 LFORTRAN_API char *_lfortran_get_env_variable(char *name);
+LFORTRAN_API char *_lfortran_get_environment_variable(char *name);
 LFORTRAN_API int _lfortran_exec_command(char *cmd);
 
 LFORTRAN_API char* _lcompilers_string_format_fortran(int count, const char* format, ...);
