@@ -58,6 +58,9 @@ struct PassOptions {
     bool tree = false;
     bool with_intrinsic_mods = false;
     bool c_mangling = false;
+    bool openmp = false;
+    bool experimental_simplifier = false;
+    bool enable_gpu_offloading = false;
 };
 
 struct CompilerOptions {
@@ -81,6 +84,13 @@ struct CompilerOptions {
     bool visualize = false;
     bool fast = false;
     bool openmp = false;
+    std::string openmp_lib_dir = "";
+    bool lookup_name = false;
+    bool rename_symbol = false;
+    std::string line = "";
+    std::string column = "";
+    bool continue_compilation = false;
+    bool semantics_only = false;
     bool generate_object_code = false;
     bool no_warnings = false;
     bool disable_style = false;
