@@ -5,13 +5,17 @@ type :: c_ptr
     integer ptr
 end type
 
+type :: c_funptr
+    integer ptr
+end type
+
 integer, parameter :: c_int8_t = 1
 integer, parameter :: c_int16_t = 2
 integer, parameter :: c_int32_t = 4
 integer, parameter :: c_int64_t = 8
 integer, parameter :: c_int = 4
 integer, parameter :: c_short = 2
-integer, parameter :: c_long = 4
+integer, parameter :: c_long = 8
 integer, parameter :: c_long_long = 8
 integer, parameter :: c_size_t = 8
 integer, parameter :: c_float = 4
@@ -20,7 +24,7 @@ integer, parameter :: c_long_double = -1
 integer, parameter :: c_float_complex = 4
 integer, parameter :: c_double_complex = 8
 integer, parameter :: c_long_double_complex = -1
-integer, parameter :: c_bool = 4
+integer, parameter :: c_bool = 1
 integer, parameter :: c_char = 1
 character(len=1), parameter :: c_null_char = char(0)
 type(c_ptr), parameter :: c_null_ptr = c_ptr(0)
@@ -45,6 +49,13 @@ interface
     !type(*), intent(in) :: x
     integer, intent(in) :: x
     end function
+
+    !type(c_funptr) function c_funloc(x)
+    integer function c_funloc(x)
+    import c_funptr
+    !type(*), intent(in) :: x
+    integer, intent(in) :: x
+    end function
 end interface
 
 end module
